// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.WindowsAzure.Management.Compute.Models;

namespace Microsoft.WindowsAzure.Management.Compute.Models
{
    /// <summary>
    /// Parameters supplied to the Create Deployment operation.
    /// </summary>
    public partial class DeploymentCreateParameters
    {
        private string _configuration;
        
        /// <summary>
        /// Required. The service configuration file for the deployment.
        /// </summary>
        public string Configuration
        {
            get { return this._configuration; }
            set { this._configuration = value; }
        }
        
        private IDictionary<string, string> _extendedProperties;
        
        /// <summary>
        /// Optional. Represents the name of an extended deployment property.
        /// Each extended property must have a defined name and a value. You
        /// can have a maximum of 25 extended property name/value pairs. The
        /// maximum length of the name element is 64 characters, only
        /// alphanumeric characters and underscores are valid in the name, and
        /// the name must start with a letter. Attempting to use other
        /// characters, starting the name with a non-letter character, or
        /// entering a name that is identical to that of another extended
        /// property owned by the same hosted service will result in a status
        /// code 400 (Bad Request) error.
        /// </summary>
        public IDictionary<string, string> ExtendedProperties
        {
            get { return this._extendedProperties; }
            set { this._extendedProperties = value; }
        }
        
        private ExtensionConfiguration _extensionConfiguration;
        
        /// <summary>
        /// Optional. Represents an extension that is added to the cloud
        /// service. In Azure, a process can run as an extension of a cloud
        /// service. For example, Remote Desktop Access or the Azure
        /// Diagnostics Agent can run as extensions to the cloud service. You
        /// must add an extension to the cloud service by using Add Extension
        /// before it can be added to the deployment.
        /// </summary>
        public ExtensionConfiguration ExtensionConfiguration
        {
            get { return this._extensionConfiguration; }
            set { this._extensionConfiguration = value; }
        }
        
        private string _label;
        
        /// <summary>
        /// Required. A name for the hosted service. The name can be up to 100
        /// characters in length. It is recommended that the label be unique
        /// within the subscription. The name can be used identify the hosted
        /// service for your tracking purposes.
        /// </summary>
        public string Label
        {
            get { return this._label; }
            set { this._label = value; }
        }
        
        private string _name;
        
        /// <summary>
        /// Required. The name for the deployment. The deployment name must be
        /// unique among other deployments for the cloud service.
        /// </summary>
        public string Name
        {
            get { return this._name; }
            set { this._name = value; }
        }
        
        private Uri _packageUri;
        
        /// <summary>
        /// Required. A URL that refers to the location of the service package
        /// in the Blob service. The service package can be located either in
        /// a storage account beneath the same subscription or a Shared Access
        /// Signature (SAS) URI from any storage account. For more info about
        /// Shared Access Signatures, see Delegating Access with a Shared
        /// Access Signature (REST API) at
        /// http://msdn.microsoft.com/en-us/library/windowsazure/ee395415.aspx.
        /// </summary>
        public Uri PackageUri
        {
            get { return this._packageUri; }
            set { this._packageUri = value; }
        }
        
        private bool? _startDeployment;
        
        /// <summary>
        /// Optional. Indicates whether to start the deployment immediately
        /// after it is created. The default value is false. If false, the
        /// service model is still deployed to the virtual machines but the
        /// code is not run immediately. Instead, the service is Suspended
        /// until you call Update Deployment Status and set the status
        /// toRunning, at which time the service will be started. A deployed
        /// service still incurs charges, even if it is suspended.
        /// </summary>
        public bool? StartDeployment
        {
            get { return this._startDeployment; }
            set { this._startDeployment = value; }
        }
        
        private bool? _treatWarningsAsError;
        
        /// <summary>
        /// Optional. Indicates whether to treat package validation warnings as
        /// errors. The default value is false. If set to true, the Created
        /// Deployment operation fails if there are validation warnings on the
        /// service package.
        /// </summary>
        public bool? TreatWarningsAsError
        {
            get { return this._treatWarningsAsError; }
            set { this._treatWarningsAsError = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the DeploymentCreateParameters class.
        /// </summary>
        public DeploymentCreateParameters()
        {
            this._extendedProperties = new Dictionary<string, string>();
        }
    }
}
