// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Common;
using Microsoft.WindowsAzure.Common.Internals;
using Microsoft.WindowsAzure.Management.Monitoring.Autoscale;
using Microsoft.WindowsAzure.Management.Monitoring.Autoscale.Models;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace Microsoft.WindowsAzure.Management.Monitoring.Autoscale.Models
{
    /// <summary>
    /// Autoscale profile.
    /// </summary>
    public partial class AutoscaleProfile
    {
        private ScaleCapacity _capacity;
        
        /// <summary>
        /// Optional. The number of instances that can be used in the scaling
        /// action.
        /// </summary>
        public ScaleCapacity Capacity
        {
            get { return this._capacity; }
            set { this._capacity = value; }
        }
        
        private TimeWindow _fixedDate;
        
        /// <summary>
        /// Optional. A specific date for the scaling action. This element is
        /// not used if the Recurrence element is used.
        /// </summary>
        public TimeWindow FixedDate
        {
            get { return this._fixedDate; }
            set { this._fixedDate = value; }
        }
        
        private string _name;
        
        /// <summary>
        /// Optional. The name of the profile.
        /// </summary>
        public string Name
        {
            get { return this._name; }
            set { this._name = value; }
        }
        
        private Recurrence _recurrence;
        
        /// <summary>
        /// Optional. The frequency and schedule for the scaling action. This
        /// element is not used if the FixedDate element is used.
        /// </summary>
        public Recurrence Recurrence
        {
            get { return this._recurrence; }
            set { this._recurrence = value; }
        }
        
        private IList<ScaleRule> _rules;
        
        /// <summary>
        /// Optional. A collection of rules that provide the triggers and
        /// parameters for the scaling action.
        /// </summary>
        public IList<ScaleRule> Rules
        {
            get { return this._rules; }
            set { this._rules = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the AutoscaleProfile class.
        /// </summary>
        public AutoscaleProfile()
        {
            this._rules = new List<ScaleRule>();
        }
    }
    
    /// <summary>
    /// Autoscale setting.
    /// </summary>
    public partial class AutoscaleSetting
    {
        private bool _enabled;
        
        /// <summary>
        /// Optional.
        /// </summary>
        public bool Enabled
        {
            get { return this._enabled; }
            set { this._enabled = value; }
        }
        
        private string _name;
        
        /// <summary>
        /// Optional.
        /// </summary>
        public string Name
        {
            get { return this._name; }
            set { this._name = value; }
        }
        
        private IList<AutoscaleProfile> _profiles;
        
        /// <summary>
        /// Optional.
        /// </summary>
        public IList<AutoscaleProfile> Profiles
        {
            get { return this._profiles; }
            set { this._profiles = value; }
        }
        
        private string _targetResourceUri;
        
        /// <summary>
        /// Optional.
        /// </summary>
        public string TargetResourceUri
        {
            get { return this._targetResourceUri; }
            set { this._targetResourceUri = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the AutoscaleSetting class.
        /// </summary>
        public AutoscaleSetting()
        {
            this._profiles = new List<AutoscaleProfile>();
        }
    }
    
    /// <summary>
    /// Parameters supplied to the Create or Update autoscale setting operation.
    /// </summary>
    public partial class AutoscaleSettingCreateOrUpdateParameters
    {
        private string _location;
        
        /// <summary>
        /// Optional. The location of the setting.
        /// </summary>
        public string Location
        {
            get { return this._location; }
            set { this._location = value; }
        }
        
        private AutoscaleSetting _properties;
        
        /// <summary>
        /// Optional. The setting to create or update.
        /// </summary>
        public AutoscaleSetting Properties
        {
            get { return this._properties; }
            set { this._properties = value; }
        }
        
        private IDictionary<string, string> _tags;
        
        /// <summary>
        /// Optional. The tags of the rule.
        /// </summary>
        public IDictionary<string, string> Tags
        {
            get { return this._tags; }
            set { this._tags = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the
        /// AutoscaleSettingCreateOrUpdateParameters class.
        /// </summary>
        public AutoscaleSettingCreateOrUpdateParameters()
        {
            this._tags = new Dictionary<string, string>();
        }
    }
    
    /// <summary>
    /// A standard service response including an HTTP status code and request
    /// ID.
    /// </summary>
    public partial class AutoscaleSettingGetResponse : OperationResponse
    {
        private string _id;
        
        /// <summary>
        /// Optional. The resource id of the setting.
        /// </summary>
        public string Id
        {
            get { return this._id; }
            set { this._id = value; }
        }
        
        private string _location;
        
        /// <summary>
        /// Optional. The location of the setting.
        /// </summary>
        public string Location
        {
            get { return this._location; }
            set { this._location = value; }
        }
        
        private string _name;
        
        /// <summary>
        /// Optional. The name of the setting.
        /// </summary>
        public string Name
        {
            get { return this._name; }
            set { this._name = value; }
        }
        
        private AutoscaleSetting _properties;
        
        /// <summary>
        /// Optional. The retrieved setting.
        /// </summary>
        public AutoscaleSetting Properties
        {
            get { return this._properties; }
            set { this._properties = value; }
        }
        
        private IDictionary<string, string> _tags;
        
        /// <summary>
        /// Optional. The tags of the rule.
        /// </summary>
        public IDictionary<string, string> Tags
        {
            get { return this._tags; }
            set { this._tags = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the AutoscaleSettingGetResponse class.
        /// </summary>
        public AutoscaleSettingGetResponse()
        {
            this._tags = new Dictionary<string, string>();
        }
    }
    
    /// <summary>
    /// The List Autoscale settings operation response.
    /// </summary>
    public partial class AutoscaleSettingListResponse : OperationResponse
    {
        private AutoscaleSettingResourceCollection _autoscaleSettingResourceCollection;
        
        /// <summary>
        /// Optional. The retrieved autoscale settings.
        /// </summary>
        public AutoscaleSettingResourceCollection AutoscaleSettingResourceCollection
        {
            get { return this._autoscaleSettingResourceCollection; }
            set { this._autoscaleSettingResourceCollection = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the AutoscaleSettingListResponse
        /// class.
        /// </summary>
        public AutoscaleSettingListResponse()
        {
        }
    }
    
    /// <summary>
    /// Autoscale setting resource.
    /// </summary>
    public partial class AutoscaleSettingResource
    {
        private string _id;
        
        /// <summary>
        /// Optional. The resource id of the autoscale setting.
        /// </summary>
        public string Id
        {
            get { return this._id; }
            set { this._id = value; }
        }
        
        private string _location;
        
        /// <summary>
        /// Optional. The location of the autoscale setting.
        /// </summary>
        public string Location
        {
            get { return this._location; }
            set { this._location = value; }
        }
        
        private string _name;
        
        /// <summary>
        /// Optional. The name of the autoscale setting.
        /// </summary>
        public string Name
        {
            get { return this._name; }
            set { this._name = value; }
        }
        
        private AutoscaleSetting _properties;
        
        /// <summary>
        /// Optional. The retrieved autoscale setting.
        /// </summary>
        public AutoscaleSetting Properties
        {
            get { return this._properties; }
            set { this._properties = value; }
        }
        
        private IDictionary<string, string> _tags;
        
        /// <summary>
        /// Optional. The tags of the rule.
        /// </summary>
        public IDictionary<string, string> Tags
        {
            get { return this._tags; }
            set { this._tags = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the AutoscaleSettingResource class.
        /// </summary>
        public AutoscaleSettingResource()
        {
            this._tags = new Dictionary<string, string>();
        }
    }
    
    /// <summary>
    /// Represents collection of autoscale settings.
    /// </summary>
    public partial class AutoscaleSettingResourceCollection
    {
        private IList<AutoscaleSettingResource> _value;
        
        /// <summary>
        /// Optional. The values for the autoscale settings.
        /// </summary>
        public IList<AutoscaleSettingResource> Value
        {
            get { return this._value; }
            set { this._value = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the
        /// AutoscaleSettingResourceCollection class.
        /// </summary>
        public AutoscaleSettingResourceCollection()
        {
            this._value = new List<AutoscaleSettingResource>();
        }
    }
    
    public enum ComparisonOperationType
    {
        /// <summary>
        /// Equals comparison.
        /// </summary>
        Equals = 0,
        
        /// <summary>
        /// NotEquals comparison.
        /// </summary>
        NotEquals = 1,
        
        /// <summary>
        /// GreaterThan comparison.
        /// </summary>
        GreaterThan = 2,
        
        /// <summary>
        /// GreaterThanOrEqual comparison.
        /// </summary>
        GreaterThanOrEqual = 3,
        
        /// <summary>
        /// LessThan comparison.
        /// </summary>
        LessThan = 4,
        
        /// <summary>
        /// LessThanOrEqual comparison.
        /// </summary>
        LessThanOrEqual = 5,
    }
    
    public enum MetricStatisticType
    {
        /// <summary>
        /// Average value.
        /// </summary>
        Average = 0,
        
        /// <summary>
        /// Minimum value.
        /// </summary>
        Min = 1,
        
        /// <summary>
        /// Maximum value.
        /// </summary>
        Max = 2,
        
        /// <summary>
        /// Total value.
        /// </summary>
        Sum = 3,
    }
    
    /// <summary>
    /// A rule that provides the triggers and parameters for the scaling action.
    /// </summary>
    public partial class MetricTrigger
    {
        private string _metricName;
        
        /// <summary>
        /// Optional. The name of the metric.
        /// </summary>
        public string MetricName
        {
            get { return this._metricName; }
            set { this._metricName = value; }
        }
        
        private string _metricNamespace;
        
        /// <summary>
        /// Optional. The namespace of the metric.
        /// </summary>
        public string MetricNamespace
        {
            get { return this._metricNamespace; }
            set { this._metricNamespace = value; }
        }
        
        private string _metricResourceUri;
        
        /// <summary>
        /// Optional. The metric resource uri.
        /// </summary>
        public string MetricResourceUri
        {
            get { return this._metricResourceUri; }
            set { this._metricResourceUri = value; }
        }
        
        private ComparisonOperationType _operator;
        
        /// <summary>
        /// Optional. The operator that is used to evaluate the metric.
        /// </summary>
        public ComparisonOperationType Operator
        {
            get { return this._operator; }
            set { this._operator = value; }
        }
        
        private MetricStatisticType _statistic;
        
        /// <summary>
        /// Optional. How the metrics from multiple instances are combined.
        /// </summary>
        public MetricStatisticType Statistic
        {
            get { return this._statistic; }
            set { this._statistic = value; }
        }
        
        private double _threshold;
        
        /// <summary>
        /// Optional. The percentage of usage that triggers the scaling
        /// operation.
        /// </summary>
        public double Threshold
        {
            get { return this._threshold; }
            set { this._threshold = value; }
        }
        
        private TimeAggregationType _timeAggregation;
        
        /// <summary>
        /// Optional. How the data that is collected should be combined over
        /// time.
        /// </summary>
        public TimeAggregationType TimeAggregation
        {
            get { return this._timeAggregation; }
            set { this._timeAggregation = value; }
        }
        
        private TimeSpan _timeGrain;
        
        /// <summary>
        /// Optional. The frequency of data collection.
        /// </summary>
        public TimeSpan TimeGrain
        {
            get { return this._timeGrain; }
            set { this._timeGrain = value; }
        }
        
        private TimeSpan _timeWindow;
        
        /// <summary>
        /// Optional. The range of time in which instance data is collected.
        /// </summary>
        public TimeSpan TimeWindow
        {
            get { return this._timeWindow; }
            set { this._timeWindow = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the MetricTrigger class.
        /// </summary>
        public MetricTrigger()
        {
        }
    }
    
    /// <summary>
    /// This describes a recurrence pattern to be used when scheduling
    /// activities.
    /// </summary>
    public partial class Recurrence
    {
        private RecurrenceFrequency _frequency;
        
        /// <summary>
        /// Optional.
        /// </summary>
        public RecurrenceFrequency Frequency
        {
            get { return this._frequency; }
            set { this._frequency = value; }
        }
        
        private RecurrentSchedule _schedule;
        
        /// <summary>
        /// Optional.
        /// </summary>
        public RecurrentSchedule Schedule
        {
            get { return this._schedule; }
            set { this._schedule = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the Recurrence class.
        /// </summary>
        public Recurrence()
        {
        }
    }
    
    public enum RecurrenceFrequency
    {
        /// <summary>
        /// No recurrence.
        /// </summary>
        None = 0,
        
        /// <summary>
        /// A recurrence value measured in seconds.
        /// </summary>
        Second = 1,
        
        /// <summary>
        /// A recurrence value measured in minutes.
        /// </summary>
        Minute = 2,
        
        /// <summary>
        /// A recurrence value measured in hours.
        /// </summary>
        Hour = 3,
        
        /// <summary>
        /// A recurrence value measured in days.
        /// </summary>
        Day = 4,
        
        /// <summary>
        /// A recurrence value measured in weeks.
        /// </summary>
        Week = 5,
        
        /// <summary>
        /// A recurrence value measured in months.
        /// </summary>
        Month = 6,
        
        /// <summary>
        /// A recurrence value measured in years.
        /// </summary>
        Year = 7,
    }
    
    public partial class RecurrentSchedule
    {
        private IList<string> _days;
        
        /// <summary>
        /// Optional.
        /// </summary>
        public IList<string> Days
        {
            get { return this._days; }
            set { this._days = value; }
        }
        
        private IList<int> _hours;
        
        /// <summary>
        /// Optional.
        /// </summary>
        public IList<int> Hours
        {
            get { return this._hours; }
            set { this._hours = value; }
        }
        
        private IList<int> _minutes;
        
        /// <summary>
        /// Optional.
        /// </summary>
        public IList<int> Minutes
        {
            get { return this._minutes; }
            set { this._minutes = value; }
        }
        
        private string _timeZone;
        
        /// <summary>
        /// Optional.
        /// </summary>
        public string TimeZone
        {
            get { return this._timeZone; }
            set { this._timeZone = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the RecurrentSchedule class.
        /// </summary>
        public RecurrentSchedule()
        {
            this._days = new List<string>();
            this._hours = new List<int>();
            this._minutes = new List<int>();
        }
    }
    
    /// <summary>
    /// The action for scaling.
    /// </summary>
    public partial class ScaleAction
    {
        private TimeSpan _cooldown;
        
        /// <summary>
        /// Optional. The cooldown interval for scaling action.
        /// </summary>
        public TimeSpan Cooldown
        {
            get { return this._cooldown; }
            set { this._cooldown = value; }
        }
        
        private ScaleDirection _direction;
        
        /// <summary>
        /// Optional. The direction of scaling.
        /// </summary>
        public ScaleDirection Direction
        {
            get { return this._direction; }
            set { this._direction = value; }
        }
        
        private ScaleType _type;
        
        /// <summary>
        /// Optional. The type of scaling.
        /// </summary>
        public ScaleType Type
        {
            get { return this._type; }
            set { this._type = value; }
        }
        
        private string _value;
        
        /// <summary>
        /// Optional. The rate of scaling.
        /// </summary>
        public string Value
        {
            get { return this._value; }
            set { this._value = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the ScaleAction class.
        /// </summary>
        public ScaleAction()
        {
        }
    }
    
    /// <summary>
    /// The number of instances that can be used in the scaling action.
    /// </summary>
    public partial class ScaleCapacity
    {
        private string _default;
        
        /// <summary>
        /// Optional. The default number of instances that are available for
        /// the scaling action.
        /// </summary>
        public string Default
        {
            get { return this._default; }
            set { this._default = value; }
        }
        
        private string _maximum;
        
        /// <summary>
        /// Optional. The maximum number of instances that are available for
        /// the scaling action.
        /// </summary>
        public string Maximum
        {
            get { return this._maximum; }
            set { this._maximum = value; }
        }
        
        private string _minimum;
        
        /// <summary>
        /// Optional. The minimum number of instances that are available for
        /// the scaling action.
        /// </summary>
        public string Minimum
        {
            get { return this._minimum; }
            set { this._minimum = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the ScaleCapacity class.
        /// </summary>
        public ScaleCapacity()
        {
        }
    }
    
    public enum ScaleDirection
    {
        /// <summary>
        /// Do nothing.
        /// </summary>
        None = 0,
        
        /// <summary>
        /// Increase capacity.
        /// </summary>
        Increase = 1,
        
        /// <summary>
        /// Decrease capacity.
        /// </summary>
        Decrease = 2,
    }
    
    /// <summary>
    /// A rule that provide the triggers and parameters for the scaling action.
    /// </summary>
    public partial class ScaleRule
    {
        private MetricTrigger _metricTrigger;
        
        /// <summary>
        /// Optional. The metrics that are used to determine when a scaling
        /// action occurs.
        /// </summary>
        public MetricTrigger MetricTrigger
        {
            get { return this._metricTrigger; }
            set { this._metricTrigger = value; }
        }
        
        private ScaleAction _scaleAction;
        
        /// <summary>
        /// Optional. The parameters for the scaling action.
        /// </summary>
        public ScaleAction ScaleAction
        {
            get { return this._scaleAction; }
            set { this._scaleAction = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the ScaleRule class.
        /// </summary>
        public ScaleRule()
        {
        }
    }
    
    public enum ScaleType
    {
        /// <summary>
        /// The change size.
        /// </summary>
        ChangeSize = 0,
        
        /// <summary>
        /// The change count.
        /// </summary>
        ChangeCount = 1,
        
        /// <summary>
        /// The percent change count.
        /// </summary>
        PercentChangeCount = 2,
        
        /// <summary>
        /// The exact count.
        /// </summary>
        ExactCount = 3,
    }
    
    public enum TimeAggregationType
    {
        /// <summary>
        /// Average aggregation.
        /// </summary>
        Average = 0,
        
        /// <summary>
        /// Minimum aggregation.
        /// </summary>
        Minimum = 1,
        
        /// <summary>
        /// Maximum aggregation.
        /// </summary>
        Maximum = 2,
        
        /// <summary>
        /// Last aggregation.
        /// </summary>
        Last = 3,
        
        /// <summary>
        /// Total aggregation.
        /// </summary>
        Total = 4,
        
        /// <summary>
        /// Count aggregation.
        /// </summary>
        Count = 5,
    }
    
    /// <summary>
    /// The action for scaling.
    /// </summary>
    public partial class TimeWindow
    {
        private DateTime _end;
        
        /// <summary>
        /// Optional. The end of the time window.
        /// </summary>
        public DateTime End
        {
            get { return this._end; }
            set { this._end = value; }
        }
        
        private DateTime _start;
        
        /// <summary>
        /// Optional. The start of the time window.
        /// </summary>
        public DateTime Start
        {
            get { return this._start; }
            set { this._start = value; }
        }
        
        private string _timeZone;
        
        /// <summary>
        /// Optional. The time zone information.
        /// </summary>
        public string TimeZone
        {
            get { return this._timeZone; }
            set { this._timeZone = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the TimeWindow class.
        /// </summary>
        public TimeWindow()
        {
        }
    }
}

namespace Microsoft.WindowsAzure.Management.Monitoring.Autoscale
{
    public static partial class AutoscaleClientExtensions
    {
    }
    
    public partial interface IAutoscaleClient : IDisposable
    {
        /// <summary>
        /// Optional base uri parameter.
        /// </summary>
        Uri BaseUri
        {
            get; 
        }
        
        /// <summary>
        /// When you create a Windows Azure subscription, it is uniquely
        /// identified by a subscription ID. The subscription ID forms part of
        /// the URI for every call that you make to the Service Management
        /// API.  The Windows Azure Service ManagementAPI use mutual
        /// authentication of management certificates over SSL to ensure that
        /// a request made to the service is secure.  No anonymous requests
        /// are allowed.
        /// </summary>
        SubscriptionCloudCredentials Credentials
        {
            get; 
        }
        
        /// <summary>
        /// Operations for managing the autoscale settings.
        /// </summary>
        ISettingOperations Settings
        {
            get; 
        }
    }
    
    public partial class AutoscaleClient : ServiceClient<AutoscaleClient>, IAutoscaleClient
    {
        private Uri _baseUri;
        
        /// <summary>
        /// Optional base uri parameter.
        /// </summary>
        public Uri BaseUri
        {
            get { return this._baseUri; }
        }
        
        private SubscriptionCloudCredentials _credentials;
        
        /// <summary>
        /// When you create a Windows Azure subscription, it is uniquely
        /// identified by a subscription ID. The subscription ID forms part of
        /// the URI for every call that you make to the Service Management
        /// API.  The Windows Azure Service ManagementAPI use mutual
        /// authentication of management certificates over SSL to ensure that
        /// a request made to the service is secure.  No anonymous requests
        /// are allowed.
        /// </summary>
        public SubscriptionCloudCredentials Credentials
        {
            get { return this._credentials; }
        }
        
        private ISettingOperations _settings;
        
        /// <summary>
        /// Operations for managing the autoscale settings.
        /// </summary>
        public virtual ISettingOperations Settings
        {
            get { return this._settings; }
        }
        
        /// <summary>
        /// Initializes a new instance of the AutoscaleClient class.
        /// </summary>
        private AutoscaleClient()
            : base()
        {
            this._settings = new SettingOperations(this);
            this.HttpClient.Timeout = TimeSpan.FromSeconds(300);
        }
        
        /// <summary>
        /// Initializes a new instance of the AutoscaleClient class.
        /// </summary>
        /// <param name='credentials'>
        /// Required. When you create a Windows Azure subscription, it is
        /// uniquely identified by a subscription ID. The subscription ID
        /// forms part of the URI for every call that you make to the Service
        /// Management API.  The Windows Azure Service ManagementAPI use
        /// mutual authentication of management certificates over SSL to
        /// ensure that a request made to the service is secure.  No anonymous
        /// requests are allowed.
        /// </param>
        /// <param name='baseUri'>
        /// Required. Optional base uri parameter.
        /// </param>
        public AutoscaleClient(SubscriptionCloudCredentials credentials, Uri baseUri)
            : this()
        {
            if (credentials == null)
            {
                throw new ArgumentNullException("credentials");
            }
            if (baseUri == null)
            {
                throw new ArgumentNullException("baseUri");
            }
            this._credentials = credentials;
            this._baseUri = baseUri;
            
            this.Credentials.InitializeServiceClient(this);
        }
        
        /// <summary>
        /// Initializes a new instance of the AutoscaleClient class.
        /// </summary>
        /// <param name='credentials'>
        /// Required. When you create a Windows Azure subscription, it is
        /// uniquely identified by a subscription ID. The subscription ID
        /// forms part of the URI for every call that you make to the Service
        /// Management API.  The Windows Azure Service ManagementAPI use
        /// mutual authentication of management certificates over SSL to
        /// ensure that a request made to the service is secure.  No anonymous
        /// requests are allowed.
        /// </param>
        public AutoscaleClient(SubscriptionCloudCredentials credentials)
            : this()
        {
            if (credentials == null)
            {
                throw new ArgumentNullException("credentials");
            }
            this._credentials = credentials;
            this._baseUri = new Uri("https://management.core.windows.net");
            
            this.Credentials.InitializeServiceClient(this);
        }
    }
    
    public static partial class SettingOperationsExtensions
    {
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Autoscale.ISettingOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource name.
        /// </param>
        /// <param name='autoscaleSettingName'>
        /// Required. The autoscale setting name.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the operation.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static OperationResponse CreateOrUpdate(this ISettingOperations operations, string resourceGroupName, string autoscaleSettingName, AutoscaleSettingCreateOrUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ISettingOperations)s).CreateOrUpdateAsync(resourceGroupName, autoscaleSettingName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Autoscale.ISettingOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource name.
        /// </param>
        /// <param name='autoscaleSettingName'>
        /// Required. The autoscale setting name.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the operation.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<OperationResponse> CreateOrUpdateAsync(this ISettingOperations operations, string resourceGroupName, string autoscaleSettingName, AutoscaleSettingCreateOrUpdateParameters parameters)
        {
            return operations.CreateOrUpdateAsync(resourceGroupName, autoscaleSettingName, parameters, CancellationToken.None);
        }
        
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Autoscale.ISettingOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource name.
        /// </param>
        /// <param name='autoscaleSettingName'>
        /// Required. The autoscale setting name.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static OperationResponse Delete(this ISettingOperations operations, string resourceGroupName, string autoscaleSettingName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ISettingOperations)s).DeleteAsync(resourceGroupName, autoscaleSettingName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Autoscale.ISettingOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource name.
        /// </param>
        /// <param name='autoscaleSettingName'>
        /// Required. The autoscale setting name.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<OperationResponse> DeleteAsync(this ISettingOperations operations, string resourceGroupName, string autoscaleSettingName)
        {
            return operations.DeleteAsync(resourceGroupName, autoscaleSettingName, CancellationToken.None);
        }
        
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Autoscale.ISettingOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource name.
        /// </param>
        /// <param name='autoscaleSettingName'>
        /// Required. The autoscale setting name.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AutoscaleSettingGetResponse Get(this ISettingOperations operations, string resourceGroupName, string autoscaleSettingName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ISettingOperations)s).GetAsync(resourceGroupName, autoscaleSettingName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Autoscale.ISettingOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource name.
        /// </param>
        /// <param name='autoscaleSettingName'>
        /// Required. The autoscale setting name.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AutoscaleSettingGetResponse> GetAsync(this ISettingOperations operations, string resourceGroupName, string autoscaleSettingName)
        {
            return operations.GetAsync(resourceGroupName, autoscaleSettingName, CancellationToken.None);
        }
        
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Autoscale.ISettingOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource name.
        /// </param>
        /// <param name='targetResourceUri'>
        /// Required. The resource uri of the target of the autoscale setting.
        /// </param>
        /// <returns>
        /// The List Autoscale settings operation response.
        /// </returns>
        public static AutoscaleSettingListResponse List(this ISettingOperations operations, string resourceGroupName, string targetResourceUri)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ISettingOperations)s).ListAsync(resourceGroupName, targetResourceUri);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Autoscale.ISettingOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource name.
        /// </param>
        /// <param name='targetResourceUri'>
        /// Required. The resource uri of the target of the autoscale setting.
        /// </param>
        /// <returns>
        /// The List Autoscale settings operation response.
        /// </returns>
        public static Task<AutoscaleSettingListResponse> ListAsync(this ISettingOperations operations, string resourceGroupName, string targetResourceUri)
        {
            return operations.ListAsync(resourceGroupName, targetResourceUri, CancellationToken.None);
        }
        
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Autoscale.ISettingOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource name.
        /// </param>
        /// <param name='autoscaleSettingName'>
        /// Required. The autoscale setting name.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the operation.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static OperationResponse Update(this ISettingOperations operations, string resourceGroupName, string autoscaleSettingName, AutoscaleSettingCreateOrUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ISettingOperations)s).UpdateAsync(resourceGroupName, autoscaleSettingName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Autoscale.ISettingOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource name.
        /// </param>
        /// <param name='autoscaleSettingName'>
        /// Required. The autoscale setting name.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the operation.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<OperationResponse> UpdateAsync(this ISettingOperations operations, string resourceGroupName, string autoscaleSettingName, AutoscaleSettingCreateOrUpdateParameters parameters)
        {
            return operations.UpdateAsync(resourceGroupName, autoscaleSettingName, parameters, CancellationToken.None);
        }
    }
    
    /// <summary>
    /// Operations for managing the autoscale settings.
    /// </summary>
    public partial interface ISettingOperations
    {
        /// <param name='resourceGroupName'>
        /// The resource name.
        /// </param>
        /// <param name='autoscaleSettingName'>
        /// The autoscale setting name.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<OperationResponse> CreateOrUpdateAsync(string resourceGroupName, string autoscaleSettingName, AutoscaleSettingCreateOrUpdateParameters parameters, CancellationToken cancellationToken);
        
        /// <param name='resourceGroupName'>
        /// The resource name.
        /// </param>
        /// <param name='autoscaleSettingName'>
        /// The autoscale setting name.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<OperationResponse> DeleteAsync(string resourceGroupName, string autoscaleSettingName, CancellationToken cancellationToken);
        
        /// <param name='resourceGroupName'>
        /// The resource name.
        /// </param>
        /// <param name='autoscaleSettingName'>
        /// The autoscale setting name.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<AutoscaleSettingGetResponse> GetAsync(string resourceGroupName, string autoscaleSettingName, CancellationToken cancellationToken);
        
        /// <param name='resourceGroupName'>
        /// The resource name.
        /// </param>
        /// <param name='targetResourceUri'>
        /// The resource uri of the target of the autoscale setting.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The List Autoscale settings operation response.
        /// </returns>
        Task<AutoscaleSettingListResponse> ListAsync(string resourceGroupName, string targetResourceUri, CancellationToken cancellationToken);
        
        /// <param name='resourceGroupName'>
        /// The resource name.
        /// </param>
        /// <param name='autoscaleSettingName'>
        /// The autoscale setting name.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<OperationResponse> UpdateAsync(string resourceGroupName, string autoscaleSettingName, AutoscaleSettingCreateOrUpdateParameters parameters, CancellationToken cancellationToken);
    }
    
    /// <summary>
    /// Operations for managing the autoscale settings.
    /// </summary>
    internal partial class SettingOperations : IServiceOperations<AutoscaleClient>, ISettingOperations
    {
        /// <summary>
        /// Initializes a new instance of the SettingOperations class.
        /// </summary>
        /// <param name='client'>
        /// Reference to the service client.
        /// </param>
        internal SettingOperations(AutoscaleClient client)
        {
            this._client = client;
        }
        
        private AutoscaleClient _client;
        
        /// <summary>
        /// Gets a reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Autoscale.AutoscaleClient.
        /// </summary>
        public AutoscaleClient Client
        {
            get { return this._client; }
        }
        
        /// <param name='resourceGroupName'>
        /// Required. The resource name.
        /// </param>
        /// <param name='autoscaleSettingName'>
        /// Required. The autoscale setting name.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public async Task<OperationResponse> CreateOrUpdateAsync(string resourceGroupName, string autoscaleSettingName, AutoscaleSettingCreateOrUpdateParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (autoscaleSettingName == null)
            {
                throw new ArgumentNullException("autoscaleSettingName");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("autoscaleSettingName", autoscaleSettingName);
                tracingParameters.Add("parameters", parameters);
                Tracing.Enter(invocationId, this, "CreateOrUpdateAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/subscriptions/" + this.Client.Credentials.SubscriptionId + "/resourcegroups/" + resourceGroupName + "/providers/microsoft.insights/autoscalesettings/" + autoscaleSettingName + "?";
            url = url + "api-version=2014-04";
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Put;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject autoscaleSettingCreateOrUpdateParametersValue = new JObject();
                requestDoc = new JObject();
                requestDoc["autoscaleSettingCreateOrUpdateParameters"] = autoscaleSettingCreateOrUpdateParametersValue;
                
                if (parameters.Location != null)
                {
                    autoscaleSettingCreateOrUpdateParametersValue["location"] = parameters.Location;
                }
                
                JObject tagsDictionary = new JObject();
                if (parameters.Tags != null)
                {
                    foreach (KeyValuePair<string, string> pair in parameters.Tags)
                    {
                        string tagsKey = pair.Key;
                        string tagsValue = pair.Value;
                        tagsDictionary[tagsKey] = tagsValue;
                    }
                }
                autoscaleSettingCreateOrUpdateParametersValue["tags"] = tagsDictionary;
                
                if (parameters.Properties != null)
                {
                    JObject propertiesValue = new JObject();
                    autoscaleSettingCreateOrUpdateParametersValue["properties"] = propertiesValue;
                    
                    if (parameters.Properties.Profiles != null)
                    {
                        JArray profilesArray = new JArray();
                        foreach (AutoscaleProfile profilesItem in parameters.Properties.Profiles)
                        {
                            JObject autoscaleProfileValue = new JObject();
                            profilesArray.Add(autoscaleProfileValue);
                            
                            if (profilesItem.Name != null)
                            {
                                autoscaleProfileValue["name"] = profilesItem.Name;
                            }
                            
                            if (profilesItem.Capacity != null)
                            {
                                JObject capacityValue = new JObject();
                                autoscaleProfileValue["capacity"] = capacityValue;
                                
                                if (profilesItem.Capacity.Minimum != null)
                                {
                                    capacityValue["minimum"] = profilesItem.Capacity.Minimum;
                                }
                                
                                if (profilesItem.Capacity.Maximum != null)
                                {
                                    capacityValue["maximum"] = profilesItem.Capacity.Maximum;
                                }
                                
                                if (profilesItem.Capacity.Default != null)
                                {
                                    capacityValue["default"] = profilesItem.Capacity.Default;
                                }
                            }
                            
                            if (profilesItem.Rules != null)
                            {
                                JArray rulesArray = new JArray();
                                foreach (ScaleRule rulesItem in profilesItem.Rules)
                                {
                                    JObject scaleRuleValue = new JObject();
                                    rulesArray.Add(scaleRuleValue);
                                    
                                    if (rulesItem.MetricTrigger != null)
                                    {
                                        JObject metricTriggerValue = new JObject();
                                        scaleRuleValue["metricTrigger"] = metricTriggerValue;
                                        
                                        if (rulesItem.MetricTrigger.MetricName != null)
                                        {
                                            metricTriggerValue["metricName"] = rulesItem.MetricTrigger.MetricName;
                                        }
                                        
                                        if (rulesItem.MetricTrigger.MetricNamespace != null)
                                        {
                                            metricTriggerValue["metricNamespace"] = rulesItem.MetricTrigger.MetricNamespace;
                                        }
                                        
                                        if (rulesItem.MetricTrigger.MetricResourceUri != null)
                                        {
                                            metricTriggerValue["metricResourceUri"] = rulesItem.MetricTrigger.MetricResourceUri;
                                        }
                                        
                                        metricTriggerValue["timeGrain"] = TypeConversion.To8601String(rulesItem.MetricTrigger.TimeGrain);
                                        
                                        metricTriggerValue["statistic"] = rulesItem.MetricTrigger.Statistic.ToString();
                                        
                                        metricTriggerValue["timeWindow"] = TypeConversion.To8601String(rulesItem.MetricTrigger.TimeWindow);
                                        
                                        metricTriggerValue["timeAggregation"] = rulesItem.MetricTrigger.TimeAggregation.ToString();
                                        
                                        metricTriggerValue["operator"] = rulesItem.MetricTrigger.Operator.ToString();
                                        
                                        metricTriggerValue["threshold"] = rulesItem.MetricTrigger.Threshold;
                                    }
                                    
                                    if (rulesItem.ScaleAction != null)
                                    {
                                        JObject scaleActionValue = new JObject();
                                        scaleRuleValue["scaleAction"] = scaleActionValue;
                                        
                                        scaleActionValue["direction"] = rulesItem.ScaleAction.Direction.ToString();
                                        
                                        scaleActionValue["type"] = rulesItem.ScaleAction.Type.ToString();
                                        
                                        if (rulesItem.ScaleAction.Value != null)
                                        {
                                            scaleActionValue["value"] = rulesItem.ScaleAction.Value;
                                        }
                                        
                                        scaleActionValue["cooldown"] = TypeConversion.To8601String(rulesItem.ScaleAction.Cooldown);
                                    }
                                }
                                autoscaleProfileValue["rules"] = rulesArray;
                            }
                            
                            if (profilesItem.FixedDate != null)
                            {
                                JObject fixedDateValue = new JObject();
                                autoscaleProfileValue["fixedDate"] = fixedDateValue;
                                
                                if (profilesItem.FixedDate.TimeZone != null)
                                {
                                    fixedDateValue["timeZone"] = profilesItem.FixedDate.TimeZone;
                                }
                                
                                fixedDateValue["start"] = profilesItem.FixedDate.Start;
                                
                                fixedDateValue["end"] = profilesItem.FixedDate.End;
                            }
                            
                            if (profilesItem.Recurrence != null)
                            {
                                JObject recurrenceValue = new JObject();
                                autoscaleProfileValue["recurrence"] = recurrenceValue;
                                
                                recurrenceValue["frequency"] = profilesItem.Recurrence.Frequency.ToString();
                                
                                if (profilesItem.Recurrence.Schedule != null)
                                {
                                    JObject scheduleValue = new JObject();
                                    recurrenceValue["schedule"] = scheduleValue;
                                    
                                    if (profilesItem.Recurrence.Schedule.TimeZone != null)
                                    {
                                        scheduleValue["timeZone"] = profilesItem.Recurrence.Schedule.TimeZone;
                                    }
                                    
                                    if (profilesItem.Recurrence.Schedule.Days != null)
                                    {
                                        JArray daysArray = new JArray();
                                        foreach (string daysItem in profilesItem.Recurrence.Schedule.Days)
                                        {
                                            daysArray.Add(daysItem);
                                        }
                                        scheduleValue["days"] = daysArray;
                                    }
                                    
                                    if (profilesItem.Recurrence.Schedule.Hours != null)
                                    {
                                        JArray hoursArray = new JArray();
                                        foreach (int hoursItem in profilesItem.Recurrence.Schedule.Hours)
                                        {
                                            hoursArray.Add(hoursItem);
                                        }
                                        scheduleValue["hours"] = hoursArray;
                                    }
                                    
                                    if (profilesItem.Recurrence.Schedule.Minutes != null)
                                    {
                                        JArray minutesArray = new JArray();
                                        foreach (int minutesItem in profilesItem.Recurrence.Schedule.Minutes)
                                        {
                                            minutesArray.Add(minutesItem);
                                        }
                                        scheduleValue["minutes"] = minutesArray;
                                    }
                                }
                            }
                        }
                        propertiesValue["profiles"] = profilesArray;
                    }
                    
                    propertiesValue["enabled"] = parameters.Properties.Enabled;
                    
                    if (parameters.Properties.Name != null)
                    {
                        propertiesValue["name"] = parameters.Properties.Name;
                    }
                    
                    if (parameters.Properties.TargetResourceUri != null)
                    {
                        propertiesValue["targetResourceUri"] = parameters.Properties.TargetResourceUri;
                    }
                }
                
                requestContent = requestDoc.ToString(Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = new MediaTypeHeaderValue("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Created)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Json);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    OperationResponse result = null;
                    result = new OperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <param name='resourceGroupName'>
        /// Required. The resource name.
        /// </param>
        /// <param name='autoscaleSettingName'>
        /// Required. The autoscale setting name.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public async Task<OperationResponse> DeleteAsync(string resourceGroupName, string autoscaleSettingName, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (autoscaleSettingName == null)
            {
                throw new ArgumentNullException("autoscaleSettingName");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("autoscaleSettingName", autoscaleSettingName);
                Tracing.Enter(invocationId, this, "DeleteAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/subscriptions/" + this.Client.Credentials.SubscriptionId + "/resourcegroups/" + resourceGroupName + "/providers/microsoft.insights/autoscalesettings/" + autoscaleSettingName + "?";
            url = url + "api-version=2014-04";
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Delete;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Json);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    OperationResponse result = null;
                    result = new OperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <param name='resourceGroupName'>
        /// Required. The resource name.
        /// </param>
        /// <param name='autoscaleSettingName'>
        /// Required. The autoscale setting name.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public async Task<AutoscaleSettingGetResponse> GetAsync(string resourceGroupName, string autoscaleSettingName, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (autoscaleSettingName == null)
            {
                throw new ArgumentNullException("autoscaleSettingName");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("autoscaleSettingName", autoscaleSettingName);
                Tracing.Enter(invocationId, this, "GetAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/subscriptions/" + this.Client.Credentials.SubscriptionId + "/resourcegroups/" + resourceGroupName + "/providers/microsoft.insights/autoscalesettings/" + autoscaleSettingName + "?";
            url = url + "api-version=2014-04";
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Json);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    AutoscaleSettingGetResponse result = null;
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    result = new AutoscaleSettingGetResponse();
                    JToken responseDoc = null;
                    if (string.IsNullOrEmpty(responseContent) == false)
                    {
                        responseDoc = JToken.Parse(responseContent);
                    }
                    
                    if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                    {
                        JToken idValue = responseDoc["id"];
                        if (idValue != null && idValue.Type != JTokenType.Null)
                        {
                            string idInstance = ((string)idValue);
                            result.Id = idInstance;
                        }
                        
                        JToken nameValue = responseDoc["name"];
                        if (nameValue != null && nameValue.Type != JTokenType.Null)
                        {
                            string nameInstance = ((string)nameValue);
                            result.Name = nameInstance;
                        }
                        
                        JToken locationValue = responseDoc["location"];
                        if (locationValue != null && locationValue.Type != JTokenType.Null)
                        {
                            string locationInstance = ((string)locationValue);
                            result.Location = locationInstance;
                        }
                        
                        JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                        if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                        {
                            foreach (JProperty property in tagsSequenceElement)
                            {
                                string tagsKey = ((string)property.Name);
                                string tagsValue = ((string)property.Value);
                                result.Tags.Add(tagsKey, tagsValue);
                            }
                        }
                        
                        JToken propertiesValue = responseDoc["properties"];
                        if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                        {
                            AutoscaleSetting propertiesInstance = new AutoscaleSetting();
                            result.Properties = propertiesInstance;
                            
                            JToken profilesArray = propertiesValue["profiles"];
                            if (profilesArray != null && profilesArray.Type != JTokenType.Null)
                            {
                                foreach (JToken profilesValue in ((JArray)profilesArray))
                                {
                                    AutoscaleProfile autoscaleProfileInstance = new AutoscaleProfile();
                                    propertiesInstance.Profiles.Add(autoscaleProfileInstance);
                                    
                                    JToken nameValue2 = profilesValue["name"];
                                    if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                    {
                                        string nameInstance2 = ((string)nameValue2);
                                        autoscaleProfileInstance.Name = nameInstance2;
                                    }
                                    
                                    JToken capacityValue = profilesValue["capacity"];
                                    if (capacityValue != null && capacityValue.Type != JTokenType.Null)
                                    {
                                        ScaleCapacity capacityInstance = new ScaleCapacity();
                                        autoscaleProfileInstance.Capacity = capacityInstance;
                                        
                                        JToken minimumValue = capacityValue["minimum"];
                                        if (minimumValue != null && minimumValue.Type != JTokenType.Null)
                                        {
                                            string minimumInstance = ((string)minimumValue);
                                            capacityInstance.Minimum = minimumInstance;
                                        }
                                        
                                        JToken maximumValue = capacityValue["maximum"];
                                        if (maximumValue != null && maximumValue.Type != JTokenType.Null)
                                        {
                                            string maximumInstance = ((string)maximumValue);
                                            capacityInstance.Maximum = maximumInstance;
                                        }
                                        
                                        JToken defaultValue = capacityValue["default"];
                                        if (defaultValue != null && defaultValue.Type != JTokenType.Null)
                                        {
                                            string defaultInstance = ((string)defaultValue);
                                            capacityInstance.Default = defaultInstance;
                                        }
                                    }
                                    
                                    JToken rulesArray = profilesValue["rules"];
                                    if (rulesArray != null && rulesArray.Type != JTokenType.Null)
                                    {
                                        foreach (JToken rulesValue in ((JArray)rulesArray))
                                        {
                                            ScaleRule scaleRuleInstance = new ScaleRule();
                                            autoscaleProfileInstance.Rules.Add(scaleRuleInstance);
                                            
                                            JToken metricTriggerValue = rulesValue["metricTrigger"];
                                            if (metricTriggerValue != null && metricTriggerValue.Type != JTokenType.Null)
                                            {
                                                MetricTrigger metricTriggerInstance = new MetricTrigger();
                                                scaleRuleInstance.MetricTrigger = metricTriggerInstance;
                                                
                                                JToken metricNameValue = metricTriggerValue["metricName"];
                                                if (metricNameValue != null && metricNameValue.Type != JTokenType.Null)
                                                {
                                                    string metricNameInstance = ((string)metricNameValue);
                                                    metricTriggerInstance.MetricName = metricNameInstance;
                                                }
                                                
                                                JToken metricNamespaceValue = metricTriggerValue["metricNamespace"];
                                                if (metricNamespaceValue != null && metricNamespaceValue.Type != JTokenType.Null)
                                                {
                                                    string metricNamespaceInstance = ((string)metricNamespaceValue);
                                                    metricTriggerInstance.MetricNamespace = metricNamespaceInstance;
                                                }
                                                
                                                JToken metricResourceUriValue = metricTriggerValue["metricResourceUri"];
                                                if (metricResourceUriValue != null && metricResourceUriValue.Type != JTokenType.Null)
                                                {
                                                    string metricResourceUriInstance = ((string)metricResourceUriValue);
                                                    metricTriggerInstance.MetricResourceUri = metricResourceUriInstance;
                                                }
                                                
                                                JToken timeGrainValue = metricTriggerValue["timeGrain"];
                                                if (timeGrainValue != null && timeGrainValue.Type != JTokenType.Null)
                                                {
                                                    TimeSpan timeGrainInstance = TypeConversion.From8601TimeSpan(((string)timeGrainValue));
                                                    metricTriggerInstance.TimeGrain = timeGrainInstance;
                                                }
                                                
                                                JToken statisticValue = metricTriggerValue["statistic"];
                                                if (statisticValue != null && statisticValue.Type != JTokenType.Null)
                                                {
                                                    MetricStatisticType statisticInstance = ((MetricStatisticType)Enum.Parse(typeof(MetricStatisticType), ((string)statisticValue), true));
                                                    metricTriggerInstance.Statistic = statisticInstance;
                                                }
                                                
                                                JToken timeWindowValue = metricTriggerValue["timeWindow"];
                                                if (timeWindowValue != null && timeWindowValue.Type != JTokenType.Null)
                                                {
                                                    TimeSpan timeWindowInstance = TypeConversion.From8601TimeSpan(((string)timeWindowValue));
                                                    metricTriggerInstance.TimeWindow = timeWindowInstance;
                                                }
                                                
                                                JToken timeAggregationValue = metricTriggerValue["timeAggregation"];
                                                if (timeAggregationValue != null && timeAggregationValue.Type != JTokenType.Null)
                                                {
                                                    TimeAggregationType timeAggregationInstance = ((TimeAggregationType)Enum.Parse(typeof(TimeAggregationType), ((string)timeAggregationValue), true));
                                                    metricTriggerInstance.TimeAggregation = timeAggregationInstance;
                                                }
                                                
                                                JToken operatorValue = metricTriggerValue["operator"];
                                                if (operatorValue != null && operatorValue.Type != JTokenType.Null)
                                                {
                                                    ComparisonOperationType operatorInstance = ((ComparisonOperationType)Enum.Parse(typeof(ComparisonOperationType), ((string)operatorValue), true));
                                                    metricTriggerInstance.Operator = operatorInstance;
                                                }
                                                
                                                JToken thresholdValue = metricTriggerValue["threshold"];
                                                if (thresholdValue != null && thresholdValue.Type != JTokenType.Null)
                                                {
                                                    double thresholdInstance = ((double)thresholdValue);
                                                    metricTriggerInstance.Threshold = thresholdInstance;
                                                }
                                            }
                                            
                                            JToken scaleActionValue = rulesValue["scaleAction"];
                                            if (scaleActionValue != null && scaleActionValue.Type != JTokenType.Null)
                                            {
                                                ScaleAction scaleActionInstance = new ScaleAction();
                                                scaleRuleInstance.ScaleAction = scaleActionInstance;
                                                
                                                JToken directionValue = scaleActionValue["direction"];
                                                if (directionValue != null && directionValue.Type != JTokenType.Null)
                                                {
                                                    ScaleDirection directionInstance = ((ScaleDirection)Enum.Parse(typeof(ScaleDirection), ((string)directionValue), true));
                                                    scaleActionInstance.Direction = directionInstance;
                                                }
                                                
                                                JToken typeValue = scaleActionValue["type"];
                                                if (typeValue != null && typeValue.Type != JTokenType.Null)
                                                {
                                                    ScaleType typeInstance = ((ScaleType)Enum.Parse(typeof(ScaleType), ((string)typeValue), true));
                                                    scaleActionInstance.Type = typeInstance;
                                                }
                                                
                                                JToken valueValue = scaleActionValue["value"];
                                                if (valueValue != null && valueValue.Type != JTokenType.Null)
                                                {
                                                    string valueInstance = ((string)valueValue);
                                                    scaleActionInstance.Value = valueInstance;
                                                }
                                                
                                                JToken cooldownValue = scaleActionValue["cooldown"];
                                                if (cooldownValue != null && cooldownValue.Type != JTokenType.Null)
                                                {
                                                    TimeSpan cooldownInstance = TypeConversion.From8601TimeSpan(((string)cooldownValue));
                                                    scaleActionInstance.Cooldown = cooldownInstance;
                                                }
                                            }
                                        }
                                    }
                                    
                                    JToken fixedDateValue = profilesValue["fixedDate"];
                                    if (fixedDateValue != null && fixedDateValue.Type != JTokenType.Null)
                                    {
                                        TimeWindow fixedDateInstance = new TimeWindow();
                                        autoscaleProfileInstance.FixedDate = fixedDateInstance;
                                        
                                        JToken timeZoneValue = fixedDateValue["timeZone"];
                                        if (timeZoneValue != null && timeZoneValue.Type != JTokenType.Null)
                                        {
                                            string timeZoneInstance = ((string)timeZoneValue);
                                            fixedDateInstance.TimeZone = timeZoneInstance;
                                        }
                                        
                                        JToken startValue = fixedDateValue["start"];
                                        if (startValue != null && startValue.Type != JTokenType.Null)
                                        {
                                            DateTime startInstance = ((DateTime)startValue);
                                            fixedDateInstance.Start = startInstance;
                                        }
                                        
                                        JToken endValue = fixedDateValue["end"];
                                        if (endValue != null && endValue.Type != JTokenType.Null)
                                        {
                                            DateTime endInstance = ((DateTime)endValue);
                                            fixedDateInstance.End = endInstance;
                                        }
                                    }
                                    
                                    JToken recurrenceValue = profilesValue["recurrence"];
                                    if (recurrenceValue != null && recurrenceValue.Type != JTokenType.Null)
                                    {
                                        Recurrence recurrenceInstance = new Recurrence();
                                        autoscaleProfileInstance.Recurrence = recurrenceInstance;
                                        
                                        JToken frequencyValue = recurrenceValue["frequency"];
                                        if (frequencyValue != null && frequencyValue.Type != JTokenType.Null)
                                        {
                                            RecurrenceFrequency frequencyInstance = ((RecurrenceFrequency)Enum.Parse(typeof(RecurrenceFrequency), ((string)frequencyValue), true));
                                            recurrenceInstance.Frequency = frequencyInstance;
                                        }
                                        
                                        JToken scheduleValue = recurrenceValue["schedule"];
                                        if (scheduleValue != null && scheduleValue.Type != JTokenType.Null)
                                        {
                                            RecurrentSchedule scheduleInstance = new RecurrentSchedule();
                                            recurrenceInstance.Schedule = scheduleInstance;
                                            
                                            JToken timeZoneValue2 = scheduleValue["timeZone"];
                                            if (timeZoneValue2 != null && timeZoneValue2.Type != JTokenType.Null)
                                            {
                                                string timeZoneInstance2 = ((string)timeZoneValue2);
                                                scheduleInstance.TimeZone = timeZoneInstance2;
                                            }
                                            
                                            JToken daysArray = scheduleValue["days"];
                                            if (daysArray != null && daysArray.Type != JTokenType.Null)
                                            {
                                                foreach (JToken daysValue in ((JArray)daysArray))
                                                {
                                                    scheduleInstance.Days.Add(((string)daysValue));
                                                }
                                            }
                                            
                                            JToken hoursArray = scheduleValue["hours"];
                                            if (hoursArray != null && hoursArray.Type != JTokenType.Null)
                                            {
                                                foreach (JToken hoursValue in ((JArray)hoursArray))
                                                {
                                                    scheduleInstance.Hours.Add(((int)hoursValue));
                                                }
                                            }
                                            
                                            JToken minutesArray = scheduleValue["minutes"];
                                            if (minutesArray != null && minutesArray.Type != JTokenType.Null)
                                            {
                                                foreach (JToken minutesValue in ((JArray)minutesArray))
                                                {
                                                    scheduleInstance.Minutes.Add(((int)minutesValue));
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                            
                            JToken enabledValue = propertiesValue["enabled"];
                            if (enabledValue != null && enabledValue.Type != JTokenType.Null)
                            {
                                bool enabledInstance = ((bool)enabledValue);
                                propertiesInstance.Enabled = enabledInstance;
                            }
                            
                            JToken nameValue3 = propertiesValue["name"];
                            if (nameValue3 != null && nameValue3.Type != JTokenType.Null)
                            {
                                string nameInstance3 = ((string)nameValue3);
                                propertiesInstance.Name = nameInstance3;
                            }
                            
                            JToken targetResourceUriValue = propertiesValue["targetResourceUri"];
                            if (targetResourceUriValue != null && targetResourceUriValue.Type != JTokenType.Null)
                            {
                                string targetResourceUriInstance = ((string)targetResourceUriValue);
                                propertiesInstance.TargetResourceUri = targetResourceUriInstance;
                            }
                        }
                    }
                    
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <param name='resourceGroupName'>
        /// Required. The resource name.
        /// </param>
        /// <param name='targetResourceUri'>
        /// Required. The resource uri of the target of the autoscale setting.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The List Autoscale settings operation response.
        /// </returns>
        public async Task<AutoscaleSettingListResponse> ListAsync(string resourceGroupName, string targetResourceUri, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("targetResourceUri", targetResourceUri);
                Tracing.Enter(invocationId, this, "ListAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/subscriptions/" + this.Client.Credentials.SubscriptionId + "/resourcegroups/" + resourceGroupName + "/providers/microsoft.insights/autoscalesettings?";
            url = url + "api-version=2014-04";
            if (targetResourceUri != null)
            {
                url = url + "&$filter=targetResourceUri eq " + Uri.EscapeUriString(targetResourceUri);
            }
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Json);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    AutoscaleSettingListResponse result = null;
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    result = new AutoscaleSettingListResponse();
                    JToken responseDoc = null;
                    if (string.IsNullOrEmpty(responseContent) == false)
                    {
                        responseDoc = JToken.Parse(responseContent);
                    }
                    
                    if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                    {
                        AutoscaleSettingResourceCollection autoscaleSettingResourceCollectionInstance = new AutoscaleSettingResourceCollection();
                        result.AutoscaleSettingResourceCollection = autoscaleSettingResourceCollectionInstance;
                        
                        JToken valueArray = responseDoc["value"];
                        if (valueArray != null && valueArray.Type != JTokenType.Null)
                        {
                            foreach (JToken valueValue in ((JArray)valueArray))
                            {
                                AutoscaleSettingResource autoscaleSettingResourceInstance = new AutoscaleSettingResource();
                                autoscaleSettingResourceCollectionInstance.Value.Add(autoscaleSettingResourceInstance);
                                
                                JToken idValue = valueValue["id"];
                                if (idValue != null && idValue.Type != JTokenType.Null)
                                {
                                    string idInstance = ((string)idValue);
                                    autoscaleSettingResourceInstance.Id = idInstance;
                                }
                                
                                JToken nameValue = valueValue["name"];
                                if (nameValue != null && nameValue.Type != JTokenType.Null)
                                {
                                    string nameInstance = ((string)nameValue);
                                    autoscaleSettingResourceInstance.Name = nameInstance;
                                }
                                
                                JToken locationValue = valueValue["location"];
                                if (locationValue != null && locationValue.Type != JTokenType.Null)
                                {
                                    string locationInstance = ((string)locationValue);
                                    autoscaleSettingResourceInstance.Location = locationInstance;
                                }
                                
                                JToken tagsSequenceElement = ((JToken)valueValue["tags"]);
                                if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                                {
                                    foreach (JProperty property in tagsSequenceElement)
                                    {
                                        string tagsKey = ((string)property.Name);
                                        string tagsValue = ((string)property.Value);
                                        autoscaleSettingResourceInstance.Tags.Add(tagsKey, tagsValue);
                                    }
                                }
                                
                                JToken propertiesValue = valueValue["properties"];
                                if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                                {
                                    AutoscaleSetting propertiesInstance = new AutoscaleSetting();
                                    autoscaleSettingResourceInstance.Properties = propertiesInstance;
                                    
                                    JToken profilesArray = propertiesValue["profiles"];
                                    if (profilesArray != null && profilesArray.Type != JTokenType.Null)
                                    {
                                        foreach (JToken profilesValue in ((JArray)profilesArray))
                                        {
                                            AutoscaleProfile autoscaleProfileInstance = new AutoscaleProfile();
                                            propertiesInstance.Profiles.Add(autoscaleProfileInstance);
                                            
                                            JToken nameValue2 = profilesValue["name"];
                                            if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                            {
                                                string nameInstance2 = ((string)nameValue2);
                                                autoscaleProfileInstance.Name = nameInstance2;
                                            }
                                            
                                            JToken capacityValue = profilesValue["capacity"];
                                            if (capacityValue != null && capacityValue.Type != JTokenType.Null)
                                            {
                                                ScaleCapacity capacityInstance = new ScaleCapacity();
                                                autoscaleProfileInstance.Capacity = capacityInstance;
                                                
                                                JToken minimumValue = capacityValue["minimum"];
                                                if (minimumValue != null && minimumValue.Type != JTokenType.Null)
                                                {
                                                    string minimumInstance = ((string)minimumValue);
                                                    capacityInstance.Minimum = minimumInstance;
                                                }
                                                
                                                JToken maximumValue = capacityValue["maximum"];
                                                if (maximumValue != null && maximumValue.Type != JTokenType.Null)
                                                {
                                                    string maximumInstance = ((string)maximumValue);
                                                    capacityInstance.Maximum = maximumInstance;
                                                }
                                                
                                                JToken defaultValue = capacityValue["default"];
                                                if (defaultValue != null && defaultValue.Type != JTokenType.Null)
                                                {
                                                    string defaultInstance = ((string)defaultValue);
                                                    capacityInstance.Default = defaultInstance;
                                                }
                                            }
                                            
                                            JToken rulesArray = profilesValue["rules"];
                                            if (rulesArray != null && rulesArray.Type != JTokenType.Null)
                                            {
                                                foreach (JToken rulesValue in ((JArray)rulesArray))
                                                {
                                                    ScaleRule scaleRuleInstance = new ScaleRule();
                                                    autoscaleProfileInstance.Rules.Add(scaleRuleInstance);
                                                    
                                                    JToken metricTriggerValue = rulesValue["metricTrigger"];
                                                    if (metricTriggerValue != null && metricTriggerValue.Type != JTokenType.Null)
                                                    {
                                                        MetricTrigger metricTriggerInstance = new MetricTrigger();
                                                        scaleRuleInstance.MetricTrigger = metricTriggerInstance;
                                                        
                                                        JToken metricNameValue = metricTriggerValue["metricName"];
                                                        if (metricNameValue != null && metricNameValue.Type != JTokenType.Null)
                                                        {
                                                            string metricNameInstance = ((string)metricNameValue);
                                                            metricTriggerInstance.MetricName = metricNameInstance;
                                                        }
                                                        
                                                        JToken metricNamespaceValue = metricTriggerValue["metricNamespace"];
                                                        if (metricNamespaceValue != null && metricNamespaceValue.Type != JTokenType.Null)
                                                        {
                                                            string metricNamespaceInstance = ((string)metricNamespaceValue);
                                                            metricTriggerInstance.MetricNamespace = metricNamespaceInstance;
                                                        }
                                                        
                                                        JToken metricResourceUriValue = metricTriggerValue["metricResourceUri"];
                                                        if (metricResourceUriValue != null && metricResourceUriValue.Type != JTokenType.Null)
                                                        {
                                                            string metricResourceUriInstance = ((string)metricResourceUriValue);
                                                            metricTriggerInstance.MetricResourceUri = metricResourceUriInstance;
                                                        }
                                                        
                                                        JToken timeGrainValue = metricTriggerValue["timeGrain"];
                                                        if (timeGrainValue != null && timeGrainValue.Type != JTokenType.Null)
                                                        {
                                                            TimeSpan timeGrainInstance = TypeConversion.From8601TimeSpan(((string)timeGrainValue));
                                                            metricTriggerInstance.TimeGrain = timeGrainInstance;
                                                        }
                                                        
                                                        JToken statisticValue = metricTriggerValue["statistic"];
                                                        if (statisticValue != null && statisticValue.Type != JTokenType.Null)
                                                        {
                                                            MetricStatisticType statisticInstance = ((MetricStatisticType)Enum.Parse(typeof(MetricStatisticType), ((string)statisticValue), true));
                                                            metricTriggerInstance.Statistic = statisticInstance;
                                                        }
                                                        
                                                        JToken timeWindowValue = metricTriggerValue["timeWindow"];
                                                        if (timeWindowValue != null && timeWindowValue.Type != JTokenType.Null)
                                                        {
                                                            TimeSpan timeWindowInstance = TypeConversion.From8601TimeSpan(((string)timeWindowValue));
                                                            metricTriggerInstance.TimeWindow = timeWindowInstance;
                                                        }
                                                        
                                                        JToken timeAggregationValue = metricTriggerValue["timeAggregation"];
                                                        if (timeAggregationValue != null && timeAggregationValue.Type != JTokenType.Null)
                                                        {
                                                            TimeAggregationType timeAggregationInstance = ((TimeAggregationType)Enum.Parse(typeof(TimeAggregationType), ((string)timeAggregationValue), true));
                                                            metricTriggerInstance.TimeAggregation = timeAggregationInstance;
                                                        }
                                                        
                                                        JToken operatorValue = metricTriggerValue["operator"];
                                                        if (operatorValue != null && operatorValue.Type != JTokenType.Null)
                                                        {
                                                            ComparisonOperationType operatorInstance = ((ComparisonOperationType)Enum.Parse(typeof(ComparisonOperationType), ((string)operatorValue), true));
                                                            metricTriggerInstance.Operator = operatorInstance;
                                                        }
                                                        
                                                        JToken thresholdValue = metricTriggerValue["threshold"];
                                                        if (thresholdValue != null && thresholdValue.Type != JTokenType.Null)
                                                        {
                                                            double thresholdInstance = ((double)thresholdValue);
                                                            metricTriggerInstance.Threshold = thresholdInstance;
                                                        }
                                                    }
                                                    
                                                    JToken scaleActionValue = rulesValue["scaleAction"];
                                                    if (scaleActionValue != null && scaleActionValue.Type != JTokenType.Null)
                                                    {
                                                        ScaleAction scaleActionInstance = new ScaleAction();
                                                        scaleRuleInstance.ScaleAction = scaleActionInstance;
                                                        
                                                        JToken directionValue = scaleActionValue["direction"];
                                                        if (directionValue != null && directionValue.Type != JTokenType.Null)
                                                        {
                                                            ScaleDirection directionInstance = ((ScaleDirection)Enum.Parse(typeof(ScaleDirection), ((string)directionValue), true));
                                                            scaleActionInstance.Direction = directionInstance;
                                                        }
                                                        
                                                        JToken typeValue = scaleActionValue["type"];
                                                        if (typeValue != null && typeValue.Type != JTokenType.Null)
                                                        {
                                                            ScaleType typeInstance = ((ScaleType)Enum.Parse(typeof(ScaleType), ((string)typeValue), true));
                                                            scaleActionInstance.Type = typeInstance;
                                                        }
                                                        
                                                        JToken valueValue2 = scaleActionValue["value"];
                                                        if (valueValue2 != null && valueValue2.Type != JTokenType.Null)
                                                        {
                                                            string valueInstance = ((string)valueValue2);
                                                            scaleActionInstance.Value = valueInstance;
                                                        }
                                                        
                                                        JToken cooldownValue = scaleActionValue["cooldown"];
                                                        if (cooldownValue != null && cooldownValue.Type != JTokenType.Null)
                                                        {
                                                            TimeSpan cooldownInstance = TypeConversion.From8601TimeSpan(((string)cooldownValue));
                                                            scaleActionInstance.Cooldown = cooldownInstance;
                                                        }
                                                    }
                                                }
                                            }
                                            
                                            JToken fixedDateValue = profilesValue["fixedDate"];
                                            if (fixedDateValue != null && fixedDateValue.Type != JTokenType.Null)
                                            {
                                                TimeWindow fixedDateInstance = new TimeWindow();
                                                autoscaleProfileInstance.FixedDate = fixedDateInstance;
                                                
                                                JToken timeZoneValue = fixedDateValue["timeZone"];
                                                if (timeZoneValue != null && timeZoneValue.Type != JTokenType.Null)
                                                {
                                                    string timeZoneInstance = ((string)timeZoneValue);
                                                    fixedDateInstance.TimeZone = timeZoneInstance;
                                                }
                                                
                                                JToken startValue = fixedDateValue["start"];
                                                if (startValue != null && startValue.Type != JTokenType.Null)
                                                {
                                                    DateTime startInstance = ((DateTime)startValue);
                                                    fixedDateInstance.Start = startInstance;
                                                }
                                                
                                                JToken endValue = fixedDateValue["end"];
                                                if (endValue != null && endValue.Type != JTokenType.Null)
                                                {
                                                    DateTime endInstance = ((DateTime)endValue);
                                                    fixedDateInstance.End = endInstance;
                                                }
                                            }
                                            
                                            JToken recurrenceValue = profilesValue["recurrence"];
                                            if (recurrenceValue != null && recurrenceValue.Type != JTokenType.Null)
                                            {
                                                Recurrence recurrenceInstance = new Recurrence();
                                                autoscaleProfileInstance.Recurrence = recurrenceInstance;
                                                
                                                JToken frequencyValue = recurrenceValue["frequency"];
                                                if (frequencyValue != null && frequencyValue.Type != JTokenType.Null)
                                                {
                                                    RecurrenceFrequency frequencyInstance = ((RecurrenceFrequency)Enum.Parse(typeof(RecurrenceFrequency), ((string)frequencyValue), true));
                                                    recurrenceInstance.Frequency = frequencyInstance;
                                                }
                                                
                                                JToken scheduleValue = recurrenceValue["schedule"];
                                                if (scheduleValue != null && scheduleValue.Type != JTokenType.Null)
                                                {
                                                    RecurrentSchedule scheduleInstance = new RecurrentSchedule();
                                                    recurrenceInstance.Schedule = scheduleInstance;
                                                    
                                                    JToken timeZoneValue2 = scheduleValue["timeZone"];
                                                    if (timeZoneValue2 != null && timeZoneValue2.Type != JTokenType.Null)
                                                    {
                                                        string timeZoneInstance2 = ((string)timeZoneValue2);
                                                        scheduleInstance.TimeZone = timeZoneInstance2;
                                                    }
                                                    
                                                    JToken daysArray = scheduleValue["days"];
                                                    if (daysArray != null && daysArray.Type != JTokenType.Null)
                                                    {
                                                        foreach (JToken daysValue in ((JArray)daysArray))
                                                        {
                                                            scheduleInstance.Days.Add(((string)daysValue));
                                                        }
                                                    }
                                                    
                                                    JToken hoursArray = scheduleValue["hours"];
                                                    if (hoursArray != null && hoursArray.Type != JTokenType.Null)
                                                    {
                                                        foreach (JToken hoursValue in ((JArray)hoursArray))
                                                        {
                                                            scheduleInstance.Hours.Add(((int)hoursValue));
                                                        }
                                                    }
                                                    
                                                    JToken minutesArray = scheduleValue["minutes"];
                                                    if (minutesArray != null && minutesArray.Type != JTokenType.Null)
                                                    {
                                                        foreach (JToken minutesValue in ((JArray)minutesArray))
                                                        {
                                                            scheduleInstance.Minutes.Add(((int)minutesValue));
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                    
                                    JToken enabledValue = propertiesValue["enabled"];
                                    if (enabledValue != null && enabledValue.Type != JTokenType.Null)
                                    {
                                        bool enabledInstance = ((bool)enabledValue);
                                        propertiesInstance.Enabled = enabledInstance;
                                    }
                                    
                                    JToken nameValue3 = propertiesValue["name"];
                                    if (nameValue3 != null && nameValue3.Type != JTokenType.Null)
                                    {
                                        string nameInstance3 = ((string)nameValue3);
                                        propertiesInstance.Name = nameInstance3;
                                    }
                                    
                                    JToken targetResourceUriValue = propertiesValue["targetResourceUri"];
                                    if (targetResourceUriValue != null && targetResourceUriValue.Type != JTokenType.Null)
                                    {
                                        string targetResourceUriInstance = ((string)targetResourceUriValue);
                                        propertiesInstance.TargetResourceUri = targetResourceUriInstance;
                                    }
                                }
                            }
                        }
                    }
                    
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <param name='resourceGroupName'>
        /// Required. The resource name.
        /// </param>
        /// <param name='autoscaleSettingName'>
        /// Required. The autoscale setting name.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public async Task<OperationResponse> UpdateAsync(string resourceGroupName, string autoscaleSettingName, AutoscaleSettingCreateOrUpdateParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (autoscaleSettingName == null)
            {
                throw new ArgumentNullException("autoscaleSettingName");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("autoscaleSettingName", autoscaleSettingName);
                tracingParameters.Add("parameters", parameters);
                Tracing.Enter(invocationId, this, "UpdateAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/subscriptions/" + this.Client.Credentials.SubscriptionId + "/resourcegroups/" + resourceGroupName + "/providers/microsoft.insights/autoscalesettings/" + autoscaleSettingName + "?";
            url = url + "api-version=2014-04";
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = new HttpMethod("PATCH");
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject autoscaleSettingCreateOrUpdateParametersValue = new JObject();
                requestDoc = new JObject();
                requestDoc["autoscaleSettingCreateOrUpdateParameters"] = autoscaleSettingCreateOrUpdateParametersValue;
                
                if (parameters.Location != null)
                {
                    autoscaleSettingCreateOrUpdateParametersValue["location"] = parameters.Location;
                }
                
                JObject tagsDictionary = new JObject();
                if (parameters.Tags != null)
                {
                    foreach (KeyValuePair<string, string> pair in parameters.Tags)
                    {
                        string tagsKey = pair.Key;
                        string tagsValue = pair.Value;
                        tagsDictionary[tagsKey] = tagsValue;
                    }
                }
                autoscaleSettingCreateOrUpdateParametersValue["tags"] = tagsDictionary;
                
                if (parameters.Properties != null)
                {
                    JObject propertiesValue = new JObject();
                    autoscaleSettingCreateOrUpdateParametersValue["properties"] = propertiesValue;
                    
                    if (parameters.Properties.Profiles != null)
                    {
                        JArray profilesArray = new JArray();
                        foreach (AutoscaleProfile profilesItem in parameters.Properties.Profiles)
                        {
                            JObject autoscaleProfileValue = new JObject();
                            profilesArray.Add(autoscaleProfileValue);
                            
                            if (profilesItem.Name != null)
                            {
                                autoscaleProfileValue["name"] = profilesItem.Name;
                            }
                            
                            if (profilesItem.Capacity != null)
                            {
                                JObject capacityValue = new JObject();
                                autoscaleProfileValue["capacity"] = capacityValue;
                                
                                if (profilesItem.Capacity.Minimum != null)
                                {
                                    capacityValue["minimum"] = profilesItem.Capacity.Minimum;
                                }
                                
                                if (profilesItem.Capacity.Maximum != null)
                                {
                                    capacityValue["maximum"] = profilesItem.Capacity.Maximum;
                                }
                                
                                if (profilesItem.Capacity.Default != null)
                                {
                                    capacityValue["default"] = profilesItem.Capacity.Default;
                                }
                            }
                            
                            if (profilesItem.Rules != null)
                            {
                                JArray rulesArray = new JArray();
                                foreach (ScaleRule rulesItem in profilesItem.Rules)
                                {
                                    JObject scaleRuleValue = new JObject();
                                    rulesArray.Add(scaleRuleValue);
                                    
                                    if (rulesItem.MetricTrigger != null)
                                    {
                                        JObject metricTriggerValue = new JObject();
                                        scaleRuleValue["metricTrigger"] = metricTriggerValue;
                                        
                                        if (rulesItem.MetricTrigger.MetricName != null)
                                        {
                                            metricTriggerValue["metricName"] = rulesItem.MetricTrigger.MetricName;
                                        }
                                        
                                        if (rulesItem.MetricTrigger.MetricNamespace != null)
                                        {
                                            metricTriggerValue["metricNamespace"] = rulesItem.MetricTrigger.MetricNamespace;
                                        }
                                        
                                        if (rulesItem.MetricTrigger.MetricResourceUri != null)
                                        {
                                            metricTriggerValue["metricResourceUri"] = rulesItem.MetricTrigger.MetricResourceUri;
                                        }
                                        
                                        metricTriggerValue["timeGrain"] = TypeConversion.To8601String(rulesItem.MetricTrigger.TimeGrain);
                                        
                                        metricTriggerValue["statistic"] = rulesItem.MetricTrigger.Statistic.ToString();
                                        
                                        metricTriggerValue["timeWindow"] = TypeConversion.To8601String(rulesItem.MetricTrigger.TimeWindow);
                                        
                                        metricTriggerValue["timeAggregation"] = rulesItem.MetricTrigger.TimeAggregation.ToString();
                                        
                                        metricTriggerValue["operator"] = rulesItem.MetricTrigger.Operator.ToString();
                                        
                                        metricTriggerValue["threshold"] = rulesItem.MetricTrigger.Threshold;
                                    }
                                    
                                    if (rulesItem.ScaleAction != null)
                                    {
                                        JObject scaleActionValue = new JObject();
                                        scaleRuleValue["scaleAction"] = scaleActionValue;
                                        
                                        scaleActionValue["direction"] = rulesItem.ScaleAction.Direction.ToString();
                                        
                                        scaleActionValue["type"] = rulesItem.ScaleAction.Type.ToString();
                                        
                                        if (rulesItem.ScaleAction.Value != null)
                                        {
                                            scaleActionValue["value"] = rulesItem.ScaleAction.Value;
                                        }
                                        
                                        scaleActionValue["cooldown"] = TypeConversion.To8601String(rulesItem.ScaleAction.Cooldown);
                                    }
                                }
                                autoscaleProfileValue["rules"] = rulesArray;
                            }
                            
                            if (profilesItem.FixedDate != null)
                            {
                                JObject fixedDateValue = new JObject();
                                autoscaleProfileValue["fixedDate"] = fixedDateValue;
                                
                                if (profilesItem.FixedDate.TimeZone != null)
                                {
                                    fixedDateValue["timeZone"] = profilesItem.FixedDate.TimeZone;
                                }
                                
                                fixedDateValue["start"] = profilesItem.FixedDate.Start;
                                
                                fixedDateValue["end"] = profilesItem.FixedDate.End;
                            }
                            
                            if (profilesItem.Recurrence != null)
                            {
                                JObject recurrenceValue = new JObject();
                                autoscaleProfileValue["recurrence"] = recurrenceValue;
                                
                                recurrenceValue["frequency"] = profilesItem.Recurrence.Frequency.ToString();
                                
                                if (profilesItem.Recurrence.Schedule != null)
                                {
                                    JObject scheduleValue = new JObject();
                                    recurrenceValue["schedule"] = scheduleValue;
                                    
                                    if (profilesItem.Recurrence.Schedule.TimeZone != null)
                                    {
                                        scheduleValue["timeZone"] = profilesItem.Recurrence.Schedule.TimeZone;
                                    }
                                    
                                    if (profilesItem.Recurrence.Schedule.Days != null)
                                    {
                                        JArray daysArray = new JArray();
                                        foreach (string daysItem in profilesItem.Recurrence.Schedule.Days)
                                        {
                                            daysArray.Add(daysItem);
                                        }
                                        scheduleValue["days"] = daysArray;
                                    }
                                    
                                    if (profilesItem.Recurrence.Schedule.Hours != null)
                                    {
                                        JArray hoursArray = new JArray();
                                        foreach (int hoursItem in profilesItem.Recurrence.Schedule.Hours)
                                        {
                                            hoursArray.Add(hoursItem);
                                        }
                                        scheduleValue["hours"] = hoursArray;
                                    }
                                    
                                    if (profilesItem.Recurrence.Schedule.Minutes != null)
                                    {
                                        JArray minutesArray = new JArray();
                                        foreach (int minutesItem in profilesItem.Recurrence.Schedule.Minutes)
                                        {
                                            minutesArray.Add(minutesItem);
                                        }
                                        scheduleValue["minutes"] = minutesArray;
                                    }
                                }
                            }
                        }
                        propertiesValue["profiles"] = profilesArray;
                    }
                    
                    propertiesValue["enabled"] = parameters.Properties.Enabled;
                    
                    if (parameters.Properties.Name != null)
                    {
                        propertiesValue["name"] = parameters.Properties.Name;
                    }
                    
                    if (parameters.Properties.TargetResourceUri != null)
                    {
                        propertiesValue["targetResourceUri"] = parameters.Properties.TargetResourceUri;
                    }
                }
                
                requestContent = requestDoc.ToString(Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = new MediaTypeHeaderValue("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Created)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Json);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    OperationResponse result = null;
                    result = new OperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
    }
}
