// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Xml.Linq;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Common;
using Microsoft.WindowsAzure.Common.Internals;
using Microsoft.WindowsAzure.Management.ServiceBus;
using Microsoft.WindowsAzure.Management.ServiceBus.Models;

namespace Microsoft.WindowsAzure.Management.ServiceBus
{
    /// <summary>
    /// The Service Bus Management API includes operations for managing Service
    /// Bus queues.
    /// </summary>
    internal partial class QueueOperations : IServiceOperations<ServiceBusManagementClient>, Microsoft.WindowsAzure.Management.ServiceBus.IQueueOperations
    {
        /// <summary>
        /// Initializes a new instance of the QueueOperations class.
        /// </summary>
        /// <param name='client'>
        /// Reference to the service client.
        /// </param>
        internal QueueOperations(ServiceBusManagementClient client)
        {
            this._client = client;
        }
        
        private ServiceBusManagementClient _client;
        
        /// <summary>
        /// Gets a reference to the
        /// Microsoft.WindowsAzure.Management.ServiceBus.ServiceBusManagementClient.
        /// </summary>
        public ServiceBusManagementClient Client
        {
            get { return this._client; }
        }
        
        /// <summary>
        /// Creates a new queue. Once created, this queue’s resource manifest
        /// is immutable. This operation is idempotent. Repeating the create
        /// call, after a queue with same name has been created successfully,
        /// will result in a 409 Conflict error message.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/jj856295.aspx
        /// for more information)
        /// </summary>
        /// <param name='namespaceName'>
        /// Required. The namespace name.
        /// </param>
        /// <param name='queue'>
        /// Required. The service bus queue.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A response to a request for a particular queue.
        /// </returns>
        public async System.Threading.Tasks.Task<Microsoft.WindowsAzure.Management.ServiceBus.Models.ServiceBusQueueResponse> CreateAsync(string namespaceName, ServiceBusQueueCreateParameters queue, CancellationToken cancellationToken)
        {
            // Validate
            if (namespaceName == null)
            {
                throw new ArgumentNullException("namespaceName");
            }
            if (queue == null)
            {
                throw new ArgumentNullException("queue");
            }
            if (queue.Name == null)
            {
                throw new ArgumentNullException("queue.Name");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("namespaceName", namespaceName);
                tracingParameters.Add("queue", queue);
                Tracing.Enter(invocationId, this, "CreateAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId + "/services/servicebus/namespaces/" + namespaceName + "/queues/" + queue.Name + "/";
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Put;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("type", "entry");
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                httpRequest.Headers.Add("x-process-at", "ServiceBus");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                XDocument requestDoc = new XDocument();
                
                XElement entryElement = new XElement(XName.Get("entry", "http://www.w3.org/2005/Atom"));
                requestDoc.Add(entryElement);
                
                XElement contentElement = new XElement(XName.Get("content", "http://www.w3.org/2005/Atom"));
                entryElement.Add(contentElement);
                
                XAttribute typeAttribute = new XAttribute(XName.Get("type", ""), "");
                typeAttribute.Value = "application/atom+xml;type=entry;charset=utf-8";
                contentElement.Add(typeAttribute);
                
                XElement queueDescriptionElement = new XElement(XName.Get("QueueDescription", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                contentElement.Add(queueDescriptionElement);
                
                if (queue.LockDuration != null)
                {
                    XElement lockDurationElement = new XElement(XName.Get("LockDuration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    lockDurationElement.Value = queue.LockDuration;
                    queueDescriptionElement.Add(lockDurationElement);
                }
                
                XElement maxSizeInMegabytesElement = new XElement(XName.Get("MaxSizeInMegabytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                maxSizeInMegabytesElement.Value = queue.MaxSizeInMegabytes.ToString();
                queueDescriptionElement.Add(maxSizeInMegabytesElement);
                
                XElement requiresDuplicateDetectionElement = new XElement(XName.Get("RequiresDuplicateDetection", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                requiresDuplicateDetectionElement.Value = queue.RequiresDuplicateDetection.ToString().ToLower();
                queueDescriptionElement.Add(requiresDuplicateDetectionElement);
                
                XElement requiresSessionElement = new XElement(XName.Get("RequiresSession", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                requiresSessionElement.Value = queue.RequiresSession.ToString().ToLower();
                queueDescriptionElement.Add(requiresSessionElement);
                
                if (queue.DefaultMessageTimeToLive != null)
                {
                    XElement defaultMessageTimeToLiveElement = new XElement(XName.Get("DefaultMessageTimeToLive", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    defaultMessageTimeToLiveElement.Value = queue.DefaultMessageTimeToLive;
                    queueDescriptionElement.Add(defaultMessageTimeToLiveElement);
                }
                
                XElement deadLetteringOnMessageExpirationElement = new XElement(XName.Get("DeadLetteringOnMessageExpiration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                deadLetteringOnMessageExpirationElement.Value = queue.DeadLetteringOnMessageExpiration.ToString().ToLower();
                queueDescriptionElement.Add(deadLetteringOnMessageExpirationElement);
                
                if (queue.DuplicateDetectionHistoryTimeWindow != null)
                {
                    XElement duplicateDetectionHistoryTimeWindowElement = new XElement(XName.Get("DuplicateDetectionHistoryTimeWindow", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    duplicateDetectionHistoryTimeWindowElement.Value = queue.DuplicateDetectionHistoryTimeWindow;
                    queueDescriptionElement.Add(duplicateDetectionHistoryTimeWindowElement);
                }
                
                XElement enableBatchedOperationsElement = new XElement(XName.Get("EnableBatchedOperations", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                enableBatchedOperationsElement.Value = queue.EnableBatchedOperations.ToString().ToLower();
                queueDescriptionElement.Add(enableBatchedOperationsElement);
                
                XElement sizeInBytesElement = new XElement(XName.Get("SizeInBytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                sizeInBytesElement.Value = queue.SizeInBytes.ToString();
                queueDescriptionElement.Add(sizeInBytesElement);
                
                XElement messageCountElement = new XElement(XName.Get("MessageCount", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                messageCountElement.Value = queue.MessageCount.ToString();
                queueDescriptionElement.Add(messageCountElement);
                
                XElement isAnonymousAccessibleElement = new XElement(XName.Get("IsAnonymousAccessible", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                isAnonymousAccessibleElement.Value = queue.IsAnonymousAccessible.ToString().ToLower();
                queueDescriptionElement.Add(isAnonymousAccessibleElement);
                
                if (queue.AuthorizationRules != null)
                {
                    XElement authorizationRulesSequenceElement = new XElement(XName.Get("AuthorizationRules", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    foreach (ServiceBusSharedAccessAuthorizationRule authorizationRulesItem in queue.AuthorizationRules)
                    {
                        XElement authorizationRuleElement = new XElement(XName.Get("AuthorizationRule", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                        authorizationRulesSequenceElement.Add(authorizationRuleElement);
                        
                        XAttribute typeAttribute2 = new XAttribute(XName.Get("type", "http://www.w3.org/2001/XMLSchema-instance"), "");
                        typeAttribute2.Value = "SharedAccessAuthorizationRule";
                        authorizationRuleElement.Add(typeAttribute2);
                        
                        if (authorizationRulesItem.ClaimType != null)
                        {
                            XElement claimTypeElement = new XElement(XName.Get("ClaimType", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            claimTypeElement.Value = authorizationRulesItem.ClaimType;
                            authorizationRuleElement.Add(claimTypeElement);
                        }
                        
                        if (authorizationRulesItem.ClaimValue != null)
                        {
                            XElement claimValueElement = new XElement(XName.Get("ClaimValue", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            claimValueElement.Value = authorizationRulesItem.ClaimValue;
                            authorizationRuleElement.Add(claimValueElement);
                        }
                        
                        if (authorizationRulesItem.Rights != null)
                        {
                            XElement rightsSequenceElement = new XElement(XName.Get("Rights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            foreach (AccessRight rightsItem in authorizationRulesItem.Rights)
                            {
                                XElement rightsItemElement = new XElement(XName.Get("AccessRights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                rightsItemElement.Value = rightsItem.ToString();
                                rightsSequenceElement.Add(rightsItemElement);
                            }
                            authorizationRuleElement.Add(rightsSequenceElement);
                        }
                        
                        XElement createdTimeElement = new XElement(XName.Get("CreatedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                        createdTimeElement.Value = authorizationRulesItem.CreatedTime.ToString();
                        authorizationRuleElement.Add(createdTimeElement);
                        
                        if (authorizationRulesItem.KeyName != null)
                        {
                            XElement keyNameElement = new XElement(XName.Get("KeyName", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            keyNameElement.Value = authorizationRulesItem.KeyName;
                            authorizationRuleElement.Add(keyNameElement);
                        }
                        
                        XElement modifiedTimeElement = new XElement(XName.Get("ModifiedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                        modifiedTimeElement.Value = authorizationRulesItem.ModifiedTime.ToString();
                        authorizationRuleElement.Add(modifiedTimeElement);
                        
                        if (authorizationRulesItem.PrimaryKey != null)
                        {
                            XElement primaryKeyElement = new XElement(XName.Get("PrimaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            primaryKeyElement.Value = authorizationRulesItem.PrimaryKey;
                            authorizationRuleElement.Add(primaryKeyElement);
                        }
                        
                        if (authorizationRulesItem.SecondaryKey != null)
                        {
                            XElement secondaryKeyElement = new XElement(XName.Get("SecondaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            secondaryKeyElement.Value = authorizationRulesItem.SecondaryKey;
                            authorizationRuleElement.Add(secondaryKeyElement);
                        }
                    }
                    queueDescriptionElement.Add(authorizationRulesSequenceElement);
                }
                
                if (queue.Status != null)
                {
                    XElement statusElement = new XElement(XName.Get("Status", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    statusElement.Value = queue.Status;
                    queueDescriptionElement.Add(statusElement);
                }
                
                XElement supportOrderingElement = new XElement(XName.Get("SupportOrdering", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                supportOrderingElement.Value = queue.SupportOrdering.ToString().ToLower();
                queueDescriptionElement.Add(supportOrderingElement);
                
                if (queue.CountDetails != null)
                {
                    XElement countDetailsElement = new XElement(XName.Get("CountDetails", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    queueDescriptionElement.Add(countDetailsElement);
                    
                    XElement activeMessageCountElement = new XElement(XName.Get("ActiveMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                    activeMessageCountElement.Value = queue.CountDetails.ActiveMessageCount.ToString();
                    countDetailsElement.Add(activeMessageCountElement);
                    
                    XElement deadLetterMessageCountElement = new XElement(XName.Get("DeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                    deadLetterMessageCountElement.Value = queue.CountDetails.DeadLetterMessageCount.ToString();
                    countDetailsElement.Add(deadLetterMessageCountElement);
                    
                    XElement scheduledMessageCountElement = new XElement(XName.Get("ScheduledMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                    scheduledMessageCountElement.Value = queue.CountDetails.ScheduledMessageCount.ToString();
                    countDetailsElement.Add(scheduledMessageCountElement);
                    
                    XElement transferDeadLetterMessageCountElement = new XElement(XName.Get("TransferDeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                    transferDeadLetterMessageCountElement.Value = queue.CountDetails.TransferDeadLetterMessageCount.ToString();
                    countDetailsElement.Add(transferDeadLetterMessageCountElement);
                    
                    XElement transferMessageCountElement = new XElement(XName.Get("TransferMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                    transferMessageCountElement.Value = queue.CountDetails.TransferMessageCount.ToString();
                    countDetailsElement.Add(transferMessageCountElement);
                }
                
                if (queue.AutoDeleteOnIdle != null)
                {
                    XElement autoDeleteOnIdleElement = new XElement(XName.Get("AutoDeleteOnIdle", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    autoDeleteOnIdleElement.Value = queue.AutoDeleteOnIdle;
                    queueDescriptionElement.Add(autoDeleteOnIdleElement);
                }
                
                if (queue.EntityAvailabilityStatus != null)
                {
                    XElement entityAvailabilityStatusElement = new XElement(XName.Get("EntityAvailabilityStatus", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    entityAvailabilityStatusElement.Value = queue.EntityAvailabilityStatus;
                    queueDescriptionElement.Add(entityAvailabilityStatusElement);
                }
                
                requestContent = requestDoc.ToString();
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = new MediaTypeHeaderValue("application/atom+xml");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.Created)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ServiceBusQueueResponse result = null;
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    result = new ServiceBusQueueResponse();
                    XDocument responseDoc = XDocument.Parse(responseContent);
                    
                    XElement entryElement2 = responseDoc.Element(XName.Get("entry", "http://www.w3.org/2005/Atom"));
                    if (entryElement2 != null && entryElement2.IsEmpty == false)
                    {
                        XElement titleElement = entryElement2.Element(XName.Get("title", "http://www.w3.org/2005/Atom"));
                        if (titleElement != null && titleElement.IsEmpty == false)
                        {
                        }
                        
                        XElement contentElement2 = entryElement2.Element(XName.Get("content", "http://www.w3.org/2005/Atom"));
                        if (contentElement2 != null && contentElement2.IsEmpty == false)
                        {
                            XElement queueDescriptionElement2 = contentElement2.Element(XName.Get("QueueDescription", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            if (queueDescriptionElement2 != null && queueDescriptionElement2.IsEmpty == false)
                            {
                                ServiceBusQueue queueDescriptionInstance = new ServiceBusQueue();
                                result.Queue = queueDescriptionInstance;
                                
                                XElement lockDurationElement2 = queueDescriptionElement2.Element(XName.Get("LockDuration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (lockDurationElement2 != null && lockDurationElement2.IsEmpty == false)
                                {
                                    string lockDurationInstance = lockDurationElement2.Value;
                                    queueDescriptionInstance.LockDuration = lockDurationInstance;
                                }
                                
                                XElement maxSizeInMegabytesElement2 = queueDescriptionElement2.Element(XName.Get("MaxSizeInMegabytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (maxSizeInMegabytesElement2 != null && maxSizeInMegabytesElement2.IsEmpty == false)
                                {
                                    int maxSizeInMegabytesInstance = int.Parse(maxSizeInMegabytesElement2.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.MaxSizeInMegabytes = maxSizeInMegabytesInstance;
                                }
                                
                                XElement requiresDuplicateDetectionElement2 = queueDescriptionElement2.Element(XName.Get("RequiresDuplicateDetection", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (requiresDuplicateDetectionElement2 != null && requiresDuplicateDetectionElement2.IsEmpty == false)
                                {
                                    bool requiresDuplicateDetectionInstance = bool.Parse(requiresDuplicateDetectionElement2.Value);
                                    queueDescriptionInstance.RequiresDuplicateDetection = requiresDuplicateDetectionInstance;
                                }
                                
                                XElement requiresSessionElement2 = queueDescriptionElement2.Element(XName.Get("RequiresSession", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (requiresSessionElement2 != null && requiresSessionElement2.IsEmpty == false)
                                {
                                    bool requiresSessionInstance = bool.Parse(requiresSessionElement2.Value);
                                    queueDescriptionInstance.RequiresSession = requiresSessionInstance;
                                }
                                
                                XElement defaultMessageTimeToLiveElement2 = queueDescriptionElement2.Element(XName.Get("DefaultMessageTimeToLive", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (defaultMessageTimeToLiveElement2 != null && defaultMessageTimeToLiveElement2.IsEmpty == false)
                                {
                                    string defaultMessageTimeToLiveInstance = defaultMessageTimeToLiveElement2.Value;
                                    queueDescriptionInstance.DefaultMessageTimeToLive = defaultMessageTimeToLiveInstance;
                                }
                                
                                XElement deadLetteringOnMessageExpirationElement2 = queueDescriptionElement2.Element(XName.Get("DeadLetteringOnMessageExpiration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (deadLetteringOnMessageExpirationElement2 != null && deadLetteringOnMessageExpirationElement2.IsEmpty == false)
                                {
                                    bool deadLetteringOnMessageExpirationInstance = bool.Parse(deadLetteringOnMessageExpirationElement2.Value);
                                    queueDescriptionInstance.DeadLetteringOnMessageExpiration = deadLetteringOnMessageExpirationInstance;
                                }
                                
                                XElement duplicateDetectionHistoryTimeWindowElement2 = queueDescriptionElement2.Element(XName.Get("DuplicateDetectionHistoryTimeWindow", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (duplicateDetectionHistoryTimeWindowElement2 != null && duplicateDetectionHistoryTimeWindowElement2.IsEmpty == false)
                                {
                                    string duplicateDetectionHistoryTimeWindowInstance = duplicateDetectionHistoryTimeWindowElement2.Value;
                                    queueDescriptionInstance.DuplicateDetectionHistoryTimeWindow = duplicateDetectionHistoryTimeWindowInstance;
                                }
                                
                                XElement maxDeliveryCountElement = queueDescriptionElement2.Element(XName.Get("MaxDeliveryCount", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (maxDeliveryCountElement != null && maxDeliveryCountElement.IsEmpty == false)
                                {
                                    int maxDeliveryCountInstance = int.Parse(maxDeliveryCountElement.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.MaxDeliveryCount = maxDeliveryCountInstance;
                                }
                                
                                XElement enableBatchedOperationsElement2 = queueDescriptionElement2.Element(XName.Get("EnableBatchedOperations", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (enableBatchedOperationsElement2 != null && enableBatchedOperationsElement2.IsEmpty == false)
                                {
                                    bool enableBatchedOperationsInstance = bool.Parse(enableBatchedOperationsElement2.Value);
                                    queueDescriptionInstance.EnableBatchedOperations = enableBatchedOperationsInstance;
                                }
                                
                                XElement sizeInBytesElement2 = queueDescriptionElement2.Element(XName.Get("SizeInBytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (sizeInBytesElement2 != null && sizeInBytesElement2.IsEmpty == false)
                                {
                                    int sizeInBytesInstance = int.Parse(sizeInBytesElement2.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.SizeInBytes = sizeInBytesInstance;
                                }
                                
                                XElement messageCountElement2 = queueDescriptionElement2.Element(XName.Get("MessageCount", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (messageCountElement2 != null && messageCountElement2.IsEmpty == false)
                                {
                                    int messageCountInstance = int.Parse(messageCountElement2.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.MessageCount = messageCountInstance;
                                }
                                
                                XElement isAnonymousAccessibleElement2 = queueDescriptionElement2.Element(XName.Get("IsAnonymousAccessible", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (isAnonymousAccessibleElement2 != null && isAnonymousAccessibleElement2.IsEmpty == false)
                                {
                                    bool isAnonymousAccessibleInstance = bool.Parse(isAnonymousAccessibleElement2.Value);
                                    queueDescriptionInstance.IsAnonymousAccessible = isAnonymousAccessibleInstance;
                                }
                                
                                XElement authorizationRulesSequenceElement2 = queueDescriptionElement2.Element(XName.Get("AuthorizationRules", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (authorizationRulesSequenceElement2 != null && authorizationRulesSequenceElement2.IsEmpty == false)
                                {
                                    foreach (XElement authorizationRulesElement in authorizationRulesSequenceElement2.Elements(XName.Get("AuthorizationRule", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect")))
                                    {
                                        ServiceBusSharedAccessAuthorizationRule authorizationRuleInstance = new ServiceBusSharedAccessAuthorizationRule();
                                        queueDescriptionInstance.AuthorizationRules.Add(authorizationRuleInstance);
                                        
                                        XElement claimTypeElement2 = authorizationRulesElement.Element(XName.Get("ClaimType", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (claimTypeElement2 != null && claimTypeElement2.IsEmpty == false)
                                        {
                                            string claimTypeInstance = claimTypeElement2.Value;
                                            authorizationRuleInstance.ClaimType = claimTypeInstance;
                                        }
                                        
                                        XElement claimValueElement2 = authorizationRulesElement.Element(XName.Get("ClaimValue", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (claimValueElement2 != null && claimValueElement2.IsEmpty == false)
                                        {
                                            string claimValueInstance = claimValueElement2.Value;
                                            authorizationRuleInstance.ClaimValue = claimValueInstance;
                                        }
                                        
                                        XElement rightsSequenceElement2 = authorizationRulesElement.Element(XName.Get("Rights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (rightsSequenceElement2 != null && rightsSequenceElement2.IsEmpty == false)
                                        {
                                            foreach (XElement rightsElement in rightsSequenceElement2.Elements(XName.Get("AccessRights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect")))
                                            {
                                                authorizationRuleInstance.Rights.Add(((AccessRight)Enum.Parse(typeof(AccessRight), rightsElement.Value, true)));
                                            }
                                        }
                                        
                                        XElement createdTimeElement2 = authorizationRulesElement.Element(XName.Get("CreatedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (createdTimeElement2 != null && createdTimeElement2.IsEmpty == false)
                                        {
                                            DateTime createdTimeInstance = DateTime.Parse(createdTimeElement2.Value, CultureInfo.InvariantCulture);
                                            authorizationRuleInstance.CreatedTime = createdTimeInstance;
                                        }
                                        
                                        XElement keyNameElement2 = authorizationRulesElement.Element(XName.Get("KeyName", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (keyNameElement2 != null && keyNameElement2.IsEmpty == false)
                                        {
                                            string keyNameInstance = keyNameElement2.Value;
                                            authorizationRuleInstance.KeyName = keyNameInstance;
                                        }
                                        
                                        XElement modifiedTimeElement2 = authorizationRulesElement.Element(XName.Get("ModifiedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (modifiedTimeElement2 != null && modifiedTimeElement2.IsEmpty == false)
                                        {
                                            DateTime modifiedTimeInstance = DateTime.Parse(modifiedTimeElement2.Value, CultureInfo.InvariantCulture);
                                            authorizationRuleInstance.ModifiedTime = modifiedTimeInstance;
                                        }
                                        
                                        XElement primaryKeyElement2 = authorizationRulesElement.Element(XName.Get("PrimaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (primaryKeyElement2 != null && primaryKeyElement2.IsEmpty == false)
                                        {
                                            string primaryKeyInstance = primaryKeyElement2.Value;
                                            authorizationRuleInstance.PrimaryKey = primaryKeyInstance;
                                        }
                                        
                                        XElement secondaryKeyElement2 = authorizationRulesElement.Element(XName.Get("SecondaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (secondaryKeyElement2 != null && secondaryKeyElement2.IsEmpty == false)
                                        {
                                            string secondaryKeyInstance = secondaryKeyElement2.Value;
                                            authorizationRuleInstance.SecondaryKey = secondaryKeyInstance;
                                        }
                                    }
                                }
                                
                                XElement statusElement2 = queueDescriptionElement2.Element(XName.Get("Status", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (statusElement2 != null && statusElement2.IsEmpty == false)
                                {
                                    string statusInstance = statusElement2.Value;
                                    queueDescriptionInstance.Status = statusInstance;
                                }
                                
                                XElement createdAtElement = queueDescriptionElement2.Element(XName.Get("CreatedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (createdAtElement != null && createdAtElement.IsEmpty == false)
                                {
                                    DateTime createdAtInstance = DateTime.Parse(createdAtElement.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.CreatedAt = createdAtInstance;
                                }
                                
                                XElement updatedAtElement = queueDescriptionElement2.Element(XName.Get("UpdatedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (updatedAtElement != null && updatedAtElement.IsEmpty == false)
                                {
                                    DateTime updatedAtInstance = DateTime.Parse(updatedAtElement.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.UpdatedAt = updatedAtInstance;
                                }
                                
                                XElement accessedAtElement = queueDescriptionElement2.Element(XName.Get("AccessedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (accessedAtElement != null && accessedAtElement.IsEmpty == false)
                                {
                                    DateTime accessedAtInstance = DateTime.Parse(accessedAtElement.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.AccessedAt = accessedAtInstance;
                                }
                                
                                XElement supportOrderingElement2 = queueDescriptionElement2.Element(XName.Get("SupportOrdering", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (supportOrderingElement2 != null && supportOrderingElement2.IsEmpty == false)
                                {
                                    bool supportOrderingInstance = bool.Parse(supportOrderingElement2.Value);
                                    queueDescriptionInstance.SupportOrdering = supportOrderingInstance;
                                }
                                
                                XElement countDetailsElement2 = queueDescriptionElement2.Element(XName.Get("CountDetails", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (countDetailsElement2 != null && countDetailsElement2.IsEmpty == false)
                                {
                                    CountDetails countDetailsInstance = new CountDetails();
                                    queueDescriptionInstance.CountDetails = countDetailsInstance;
                                    
                                    XElement activeMessageCountElement2 = countDetailsElement2.Element(XName.Get("ActiveMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                    if (activeMessageCountElement2 != null && activeMessageCountElement2.IsEmpty == false)
                                    {
                                        int activeMessageCountInstance = int.Parse(activeMessageCountElement2.Value, CultureInfo.InvariantCulture);
                                        countDetailsInstance.ActiveMessageCount = activeMessageCountInstance;
                                    }
                                    
                                    XElement deadLetterMessageCountElement2 = countDetailsElement2.Element(XName.Get("DeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                    if (deadLetterMessageCountElement2 != null && deadLetterMessageCountElement2.IsEmpty == false)
                                    {
                                        int deadLetterMessageCountInstance = int.Parse(deadLetterMessageCountElement2.Value, CultureInfo.InvariantCulture);
                                        countDetailsInstance.DeadLetterMessageCount = deadLetterMessageCountInstance;
                                    }
                                    
                                    XElement scheduledMessageCountElement2 = countDetailsElement2.Element(XName.Get("ScheduledMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                    if (scheduledMessageCountElement2 != null && scheduledMessageCountElement2.IsEmpty == false)
                                    {
                                        int scheduledMessageCountInstance = int.Parse(scheduledMessageCountElement2.Value, CultureInfo.InvariantCulture);
                                        countDetailsInstance.ScheduledMessageCount = scheduledMessageCountInstance;
                                    }
                                    
                                    XElement transferDeadLetterMessageCountElement2 = countDetailsElement2.Element(XName.Get("TransferDeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                    if (transferDeadLetterMessageCountElement2 != null && transferDeadLetterMessageCountElement2.IsEmpty == false)
                                    {
                                        int transferDeadLetterMessageCountInstance = int.Parse(transferDeadLetterMessageCountElement2.Value, CultureInfo.InvariantCulture);
                                        countDetailsInstance.TransferDeadLetterMessageCount = transferDeadLetterMessageCountInstance;
                                    }
                                    
                                    XElement transferMessageCountElement2 = countDetailsElement2.Element(XName.Get("TransferMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                    if (transferMessageCountElement2 != null && transferMessageCountElement2.IsEmpty == false)
                                    {
                                        int transferMessageCountInstance = int.Parse(transferMessageCountElement2.Value, CultureInfo.InvariantCulture);
                                        countDetailsInstance.TransferMessageCount = transferMessageCountInstance;
                                    }
                                }
                                
                                XElement autoDeleteOnIdleElement2 = queueDescriptionElement2.Element(XName.Get("AutoDeleteOnIdle", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (autoDeleteOnIdleElement2 != null && autoDeleteOnIdleElement2.IsEmpty == false)
                                {
                                    string autoDeleteOnIdleInstance = autoDeleteOnIdleElement2.Value;
                                    queueDescriptionInstance.AutoDeleteOnIdle = autoDeleteOnIdleInstance;
                                }
                                
                                XElement entityAvailabilityStatusElement2 = queueDescriptionElement2.Element(XName.Get("EntityAvailabilityStatus", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (entityAvailabilityStatusElement2 != null && entityAvailabilityStatusElement2.IsEmpty == false)
                                {
                                    string entityAvailabilityStatusInstance = entityAvailabilityStatusElement2.Value;
                                    queueDescriptionInstance.EntityAvailabilityStatus = entityAvailabilityStatusInstance;
                                }
                            }
                        }
                    }
                    
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The queue description is an XML AtomPub document that defines the
        /// desired semantics for a subscription. The queue description
        /// contains the following properties. For more information, see the
        /// QueueDescription Properties topic.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/hh780773.aspx
        /// for more information)
        /// </summary>
        /// <param name='namespaceName'>
        /// Required. The namespace name.
        /// </param>
        /// <param name='queueName'>
        /// Required. The queue name.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A response to a request for a particular queue.
        /// </returns>
        public async System.Threading.Tasks.Task<Microsoft.WindowsAzure.Management.ServiceBus.Models.ServiceBusQueueResponse> GetAsync(string namespaceName, string queueName, CancellationToken cancellationToken)
        {
            // Validate
            if (namespaceName == null)
            {
                throw new ArgumentNullException("namespaceName");
            }
            if (queueName == null)
            {
                throw new ArgumentNullException("queueName");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("namespaceName", namespaceName);
                tracingParameters.Add("queueName", queueName);
                Tracing.Enter(invocationId, this, "GetAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId + "/services/servicebus/namespaces/" + namespaceName + "/Queues/" + queueName;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ServiceBusQueueResponse result = null;
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    result = new ServiceBusQueueResponse();
                    XDocument responseDoc = XDocument.Parse(responseContent);
                    
                    XElement entryElement = responseDoc.Element(XName.Get("entry", "http://www.w3.org/2005/Atom"));
                    if (entryElement != null && entryElement.IsEmpty == false)
                    {
                        XElement titleElement = entryElement.Element(XName.Get("title", "http://www.w3.org/2005/Atom"));
                        if (titleElement != null && titleElement.IsEmpty == false)
                        {
                        }
                        
                        XElement contentElement = entryElement.Element(XName.Get("content", "http://www.w3.org/2005/Atom"));
                        if (contentElement != null && contentElement.IsEmpty == false)
                        {
                            XElement queueDescriptionElement = contentElement.Element(XName.Get("QueueDescription", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            if (queueDescriptionElement != null && queueDescriptionElement.IsEmpty == false)
                            {
                                ServiceBusQueue queueDescriptionInstance = new ServiceBusQueue();
                                result.Queue = queueDescriptionInstance;
                                
                                XElement lockDurationElement = queueDescriptionElement.Element(XName.Get("LockDuration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (lockDurationElement != null && lockDurationElement.IsEmpty == false)
                                {
                                    string lockDurationInstance = lockDurationElement.Value;
                                    queueDescriptionInstance.LockDuration = lockDurationInstance;
                                }
                                
                                XElement maxSizeInMegabytesElement = queueDescriptionElement.Element(XName.Get("MaxSizeInMegabytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (maxSizeInMegabytesElement != null && maxSizeInMegabytesElement.IsEmpty == false)
                                {
                                    int maxSizeInMegabytesInstance = int.Parse(maxSizeInMegabytesElement.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.MaxSizeInMegabytes = maxSizeInMegabytesInstance;
                                }
                                
                                XElement requiresDuplicateDetectionElement = queueDescriptionElement.Element(XName.Get("RequiresDuplicateDetection", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (requiresDuplicateDetectionElement != null && requiresDuplicateDetectionElement.IsEmpty == false)
                                {
                                    bool requiresDuplicateDetectionInstance = bool.Parse(requiresDuplicateDetectionElement.Value);
                                    queueDescriptionInstance.RequiresDuplicateDetection = requiresDuplicateDetectionInstance;
                                }
                                
                                XElement requiresSessionElement = queueDescriptionElement.Element(XName.Get("RequiresSession", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (requiresSessionElement != null && requiresSessionElement.IsEmpty == false)
                                {
                                    bool requiresSessionInstance = bool.Parse(requiresSessionElement.Value);
                                    queueDescriptionInstance.RequiresSession = requiresSessionInstance;
                                }
                                
                                XElement defaultMessageTimeToLiveElement = queueDescriptionElement.Element(XName.Get("DefaultMessageTimeToLive", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (defaultMessageTimeToLiveElement != null && defaultMessageTimeToLiveElement.IsEmpty == false)
                                {
                                    string defaultMessageTimeToLiveInstance = defaultMessageTimeToLiveElement.Value;
                                    queueDescriptionInstance.DefaultMessageTimeToLive = defaultMessageTimeToLiveInstance;
                                }
                                
                                XElement deadLetteringOnMessageExpirationElement = queueDescriptionElement.Element(XName.Get("DeadLetteringOnMessageExpiration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (deadLetteringOnMessageExpirationElement != null && deadLetteringOnMessageExpirationElement.IsEmpty == false)
                                {
                                    bool deadLetteringOnMessageExpirationInstance = bool.Parse(deadLetteringOnMessageExpirationElement.Value);
                                    queueDescriptionInstance.DeadLetteringOnMessageExpiration = deadLetteringOnMessageExpirationInstance;
                                }
                                
                                XElement duplicateDetectionHistoryTimeWindowElement = queueDescriptionElement.Element(XName.Get("DuplicateDetectionHistoryTimeWindow", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (duplicateDetectionHistoryTimeWindowElement != null && duplicateDetectionHistoryTimeWindowElement.IsEmpty == false)
                                {
                                    string duplicateDetectionHistoryTimeWindowInstance = duplicateDetectionHistoryTimeWindowElement.Value;
                                    queueDescriptionInstance.DuplicateDetectionHistoryTimeWindow = duplicateDetectionHistoryTimeWindowInstance;
                                }
                                
                                XElement maxDeliveryCountElement = queueDescriptionElement.Element(XName.Get("MaxDeliveryCount", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (maxDeliveryCountElement != null && maxDeliveryCountElement.IsEmpty == false)
                                {
                                    int maxDeliveryCountInstance = int.Parse(maxDeliveryCountElement.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.MaxDeliveryCount = maxDeliveryCountInstance;
                                }
                                
                                XElement enableBatchedOperationsElement = queueDescriptionElement.Element(XName.Get("EnableBatchedOperations", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (enableBatchedOperationsElement != null && enableBatchedOperationsElement.IsEmpty == false)
                                {
                                    bool enableBatchedOperationsInstance = bool.Parse(enableBatchedOperationsElement.Value);
                                    queueDescriptionInstance.EnableBatchedOperations = enableBatchedOperationsInstance;
                                }
                                
                                XElement sizeInBytesElement = queueDescriptionElement.Element(XName.Get("SizeInBytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (sizeInBytesElement != null && sizeInBytesElement.IsEmpty == false)
                                {
                                    int sizeInBytesInstance = int.Parse(sizeInBytesElement.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.SizeInBytes = sizeInBytesInstance;
                                }
                                
                                XElement messageCountElement = queueDescriptionElement.Element(XName.Get("MessageCount", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (messageCountElement != null && messageCountElement.IsEmpty == false)
                                {
                                    int messageCountInstance = int.Parse(messageCountElement.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.MessageCount = messageCountInstance;
                                }
                                
                                XElement isAnonymousAccessibleElement = queueDescriptionElement.Element(XName.Get("IsAnonymousAccessible", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (isAnonymousAccessibleElement != null && isAnonymousAccessibleElement.IsEmpty == false)
                                {
                                    bool isAnonymousAccessibleInstance = bool.Parse(isAnonymousAccessibleElement.Value);
                                    queueDescriptionInstance.IsAnonymousAccessible = isAnonymousAccessibleInstance;
                                }
                                
                                XElement authorizationRulesSequenceElement = queueDescriptionElement.Element(XName.Get("AuthorizationRules", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (authorizationRulesSequenceElement != null && authorizationRulesSequenceElement.IsEmpty == false)
                                {
                                    foreach (XElement authorizationRulesElement in authorizationRulesSequenceElement.Elements(XName.Get("AuthorizationRule", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect")))
                                    {
                                        ServiceBusSharedAccessAuthorizationRule authorizationRuleInstance = new ServiceBusSharedAccessAuthorizationRule();
                                        queueDescriptionInstance.AuthorizationRules.Add(authorizationRuleInstance);
                                        
                                        XElement claimTypeElement = authorizationRulesElement.Element(XName.Get("ClaimType", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (claimTypeElement != null && claimTypeElement.IsEmpty == false)
                                        {
                                            string claimTypeInstance = claimTypeElement.Value;
                                            authorizationRuleInstance.ClaimType = claimTypeInstance;
                                        }
                                        
                                        XElement claimValueElement = authorizationRulesElement.Element(XName.Get("ClaimValue", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (claimValueElement != null && claimValueElement.IsEmpty == false)
                                        {
                                            string claimValueInstance = claimValueElement.Value;
                                            authorizationRuleInstance.ClaimValue = claimValueInstance;
                                        }
                                        
                                        XElement rightsSequenceElement = authorizationRulesElement.Element(XName.Get("Rights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (rightsSequenceElement != null && rightsSequenceElement.IsEmpty == false)
                                        {
                                            foreach (XElement rightsElement in rightsSequenceElement.Elements(XName.Get("AccessRights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect")))
                                            {
                                                authorizationRuleInstance.Rights.Add(((AccessRight)Enum.Parse(typeof(AccessRight), rightsElement.Value, true)));
                                            }
                                        }
                                        
                                        XElement createdTimeElement = authorizationRulesElement.Element(XName.Get("CreatedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (createdTimeElement != null && createdTimeElement.IsEmpty == false)
                                        {
                                            DateTime createdTimeInstance = DateTime.Parse(createdTimeElement.Value, CultureInfo.InvariantCulture);
                                            authorizationRuleInstance.CreatedTime = createdTimeInstance;
                                        }
                                        
                                        XElement keyNameElement = authorizationRulesElement.Element(XName.Get("KeyName", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (keyNameElement != null && keyNameElement.IsEmpty == false)
                                        {
                                            string keyNameInstance = keyNameElement.Value;
                                            authorizationRuleInstance.KeyName = keyNameInstance;
                                        }
                                        
                                        XElement modifiedTimeElement = authorizationRulesElement.Element(XName.Get("ModifiedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (modifiedTimeElement != null && modifiedTimeElement.IsEmpty == false)
                                        {
                                            DateTime modifiedTimeInstance = DateTime.Parse(modifiedTimeElement.Value, CultureInfo.InvariantCulture);
                                            authorizationRuleInstance.ModifiedTime = modifiedTimeInstance;
                                        }
                                        
                                        XElement primaryKeyElement = authorizationRulesElement.Element(XName.Get("PrimaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (primaryKeyElement != null && primaryKeyElement.IsEmpty == false)
                                        {
                                            string primaryKeyInstance = primaryKeyElement.Value;
                                            authorizationRuleInstance.PrimaryKey = primaryKeyInstance;
                                        }
                                        
                                        XElement secondaryKeyElement = authorizationRulesElement.Element(XName.Get("SecondaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (secondaryKeyElement != null && secondaryKeyElement.IsEmpty == false)
                                        {
                                            string secondaryKeyInstance = secondaryKeyElement.Value;
                                            authorizationRuleInstance.SecondaryKey = secondaryKeyInstance;
                                        }
                                    }
                                }
                                
                                XElement statusElement = queueDescriptionElement.Element(XName.Get("Status", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (statusElement != null && statusElement.IsEmpty == false)
                                {
                                    string statusInstance = statusElement.Value;
                                    queueDescriptionInstance.Status = statusInstance;
                                }
                                
                                XElement createdAtElement = queueDescriptionElement.Element(XName.Get("CreatedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (createdAtElement != null && createdAtElement.IsEmpty == false)
                                {
                                    DateTime createdAtInstance = DateTime.Parse(createdAtElement.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.CreatedAt = createdAtInstance;
                                }
                                
                                XElement updatedAtElement = queueDescriptionElement.Element(XName.Get("UpdatedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (updatedAtElement != null && updatedAtElement.IsEmpty == false)
                                {
                                    DateTime updatedAtInstance = DateTime.Parse(updatedAtElement.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.UpdatedAt = updatedAtInstance;
                                }
                                
                                XElement accessedAtElement = queueDescriptionElement.Element(XName.Get("AccessedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (accessedAtElement != null && accessedAtElement.IsEmpty == false)
                                {
                                    DateTime accessedAtInstance = DateTime.Parse(accessedAtElement.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.AccessedAt = accessedAtInstance;
                                }
                                
                                XElement supportOrderingElement = queueDescriptionElement.Element(XName.Get("SupportOrdering", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (supportOrderingElement != null && supportOrderingElement.IsEmpty == false)
                                {
                                    bool supportOrderingInstance = bool.Parse(supportOrderingElement.Value);
                                    queueDescriptionInstance.SupportOrdering = supportOrderingInstance;
                                }
                                
                                XElement countDetailsElement = queueDescriptionElement.Element(XName.Get("CountDetails", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (countDetailsElement != null && countDetailsElement.IsEmpty == false)
                                {
                                    CountDetails countDetailsInstance = new CountDetails();
                                    queueDescriptionInstance.CountDetails = countDetailsInstance;
                                    
                                    XElement activeMessageCountElement = countDetailsElement.Element(XName.Get("ActiveMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                    if (activeMessageCountElement != null && activeMessageCountElement.IsEmpty == false)
                                    {
                                        int activeMessageCountInstance = int.Parse(activeMessageCountElement.Value, CultureInfo.InvariantCulture);
                                        countDetailsInstance.ActiveMessageCount = activeMessageCountInstance;
                                    }
                                    
                                    XElement deadLetterMessageCountElement = countDetailsElement.Element(XName.Get("DeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                    if (deadLetterMessageCountElement != null && deadLetterMessageCountElement.IsEmpty == false)
                                    {
                                        int deadLetterMessageCountInstance = int.Parse(deadLetterMessageCountElement.Value, CultureInfo.InvariantCulture);
                                        countDetailsInstance.DeadLetterMessageCount = deadLetterMessageCountInstance;
                                    }
                                    
                                    XElement scheduledMessageCountElement = countDetailsElement.Element(XName.Get("ScheduledMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                    if (scheduledMessageCountElement != null && scheduledMessageCountElement.IsEmpty == false)
                                    {
                                        int scheduledMessageCountInstance = int.Parse(scheduledMessageCountElement.Value, CultureInfo.InvariantCulture);
                                        countDetailsInstance.ScheduledMessageCount = scheduledMessageCountInstance;
                                    }
                                    
                                    XElement transferDeadLetterMessageCountElement = countDetailsElement.Element(XName.Get("TransferDeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                    if (transferDeadLetterMessageCountElement != null && transferDeadLetterMessageCountElement.IsEmpty == false)
                                    {
                                        int transferDeadLetterMessageCountInstance = int.Parse(transferDeadLetterMessageCountElement.Value, CultureInfo.InvariantCulture);
                                        countDetailsInstance.TransferDeadLetterMessageCount = transferDeadLetterMessageCountInstance;
                                    }
                                    
                                    XElement transferMessageCountElement = countDetailsElement.Element(XName.Get("TransferMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                    if (transferMessageCountElement != null && transferMessageCountElement.IsEmpty == false)
                                    {
                                        int transferMessageCountInstance = int.Parse(transferMessageCountElement.Value, CultureInfo.InvariantCulture);
                                        countDetailsInstance.TransferMessageCount = transferMessageCountInstance;
                                    }
                                }
                                
                                XElement autoDeleteOnIdleElement = queueDescriptionElement.Element(XName.Get("AutoDeleteOnIdle", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (autoDeleteOnIdleElement != null && autoDeleteOnIdleElement.IsEmpty == false)
                                {
                                    string autoDeleteOnIdleInstance = autoDeleteOnIdleElement.Value;
                                    queueDescriptionInstance.AutoDeleteOnIdle = autoDeleteOnIdleInstance;
                                }
                                
                                XElement entityAvailabilityStatusElement = queueDescriptionElement.Element(XName.Get("EntityAvailabilityStatus", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (entityAvailabilityStatusElement != null && entityAvailabilityStatusElement.IsEmpty == false)
                                {
                                    string entityAvailabilityStatusInstance = entityAvailabilityStatusElement.Value;
                                    queueDescriptionInstance.EntityAvailabilityStatus = entityAvailabilityStatusInstance;
                                }
                            }
                        }
                    }
                    
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Gets the set of connection strings for a queue.
        /// </summary>
        /// <param name='namespaceName'>
        /// Required. The namespace name.
        /// </param>
        /// <param name='queueName'>
        /// Required. The queue name.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The set of connection details for a service bus entity.
        /// </returns>
        public async System.Threading.Tasks.Task<Microsoft.WindowsAzure.Management.ServiceBus.Models.ServiceBusConnectionDetailsResponse> GetConnectionDetailsAsync(string namespaceName, string queueName, CancellationToken cancellationToken)
        {
            // Validate
            if (namespaceName == null)
            {
                throw new ArgumentNullException("namespaceName");
            }
            if (queueName == null)
            {
                throw new ArgumentNullException("queueName");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("namespaceName", namespaceName);
                tracingParameters.Add("queueName", queueName);
                Tracing.Enter(invocationId, this, "GetConnectionDetailsAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId + "/services/servicebus/namespaces/" + namespaceName + "/Queues/" + queueName + "/ConnectionDetails";
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ServiceBusConnectionDetailsResponse result = null;
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    result = new ServiceBusConnectionDetailsResponse();
                    XDocument responseDoc = XDocument.Parse(responseContent);
                    
                    XElement feedElement = responseDoc.Element(XName.Get("feed", "http://www.w3.org/2005/Atom"));
                    if (feedElement != null && feedElement.IsEmpty == false)
                    {
                        if (feedElement != null && feedElement.IsEmpty == false)
                        {
                            foreach (XElement entriesElement in feedElement.Elements(XName.Get("entry", "http://www.w3.org/2005/Atom")))
                            {
                                ServiceBusConnectionDetail entryInstance = new ServiceBusConnectionDetail();
                                result.ConnectionDetails.Add(entryInstance);
                                
                                XElement contentElement = entriesElement.Element(XName.Get("content", "http://www.w3.org/2005/Atom"));
                                if (contentElement != null && contentElement.IsEmpty == false)
                                {
                                    XElement connectionDetailElement = contentElement.Element(XName.Get("ConnectionDetail", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (connectionDetailElement != null && connectionDetailElement.IsEmpty == false)
                                    {
                                        XElement keyNameElement = connectionDetailElement.Element(XName.Get("KeyName", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (keyNameElement != null && keyNameElement.IsEmpty == false)
                                        {
                                            string keyNameInstance = keyNameElement.Value;
                                            entryInstance.KeyName = keyNameInstance;
                                        }
                                        
                                        XElement connectionStringElement = connectionDetailElement.Element(XName.Get("ConnectionString", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (connectionStringElement != null && connectionStringElement.IsEmpty == false)
                                        {
                                            string connectionStringInstance = connectionStringElement.Value;
                                            entryInstance.ConnectionString = connectionStringInstance;
                                        }
                                        
                                        XElement authorizationTypeElement = connectionDetailElement.Element(XName.Get("AuthorizationType", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (authorizationTypeElement != null && authorizationTypeElement.IsEmpty == false)
                                        {
                                            string authorizationTypeInstance = authorizationTypeElement.Value;
                                            entryInstance.AuthorizationType = authorizationTypeInstance;
                                        }
                                        
                                        XElement rightsSequenceElement = connectionDetailElement.Element(XName.Get("Rights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (rightsSequenceElement != null && rightsSequenceElement.IsEmpty == false)
                                        {
                                            foreach (XElement rightsElement in rightsSequenceElement.Elements(XName.Get("AccessRights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect")))
                                            {
                                                entryInstance.Rights.Add(((AccessRight)Enum.Parse(typeof(AccessRight), rightsElement.Value, true)));
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                    
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Enumerates the queues in the service namespace. The result is
        /// returned in pages, each containing up to 100 queues. If the
        /// namespace contains more than 100 queues, a feed is returned that
        /// contains the first page and a next link with the URI to view the
        /// next page of data.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/hh780759.asp
        /// for more information)
        /// </summary>
        /// <param name='namespaceName'>
        /// Required. The namespace name.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A response to a request for a list of queues.
        /// </returns>
        public async System.Threading.Tasks.Task<Microsoft.WindowsAzure.Management.ServiceBus.Models.ServiceBusQueuesResponse> ListAsync(string namespaceName, CancellationToken cancellationToken)
        {
            // Validate
            if (namespaceName == null)
            {
                throw new ArgumentNullException("namespaceName");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("namespaceName", namespaceName);
                Tracing.Enter(invocationId, this, "ListAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId + "/services/servicebus/namespaces/" + namespaceName + "/Queues";
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ServiceBusQueuesResponse result = null;
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    result = new ServiceBusQueuesResponse();
                    XDocument responseDoc = XDocument.Parse(responseContent);
                    
                    XElement feedElement = responseDoc.Element(XName.Get("feed", "http://www.w3.org/2005/Atom"));
                    if (feedElement != null && feedElement.IsEmpty == false)
                    {
                        if (feedElement != null && feedElement.IsEmpty == false)
                        {
                            foreach (XElement entriesElement in feedElement.Elements(XName.Get("entry", "http://www.w3.org/2005/Atom")))
                            {
                                ServiceBusQueue entryInstance = new ServiceBusQueue();
                                result.Queues.Add(entryInstance);
                                
                                XElement titleElement = entriesElement.Element(XName.Get("title", "http://www.w3.org/2005/Atom"));
                                if (titleElement != null && titleElement.IsEmpty == false)
                                {
                                    string titleInstance = titleElement.Value;
                                    entryInstance.Name = titleInstance;
                                }
                                
                                XElement contentElement = entriesElement.Element(XName.Get("content", "http://www.w3.org/2005/Atom"));
                                if (contentElement != null && contentElement.IsEmpty == false)
                                {
                                    XElement queueDescriptionElement = contentElement.Element(XName.Get("QueueDescription", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (queueDescriptionElement != null && queueDescriptionElement.IsEmpty == false)
                                    {
                                        XElement lockDurationElement = queueDescriptionElement.Element(XName.Get("LockDuration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (lockDurationElement != null && lockDurationElement.IsEmpty == false)
                                        {
                                            string lockDurationInstance = lockDurationElement.Value;
                                            entryInstance.LockDuration = lockDurationInstance;
                                        }
                                        
                                        XElement maxSizeInMegabytesElement = queueDescriptionElement.Element(XName.Get("MaxSizeInMegabytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (maxSizeInMegabytesElement != null && maxSizeInMegabytesElement.IsEmpty == false)
                                        {
                                            int maxSizeInMegabytesInstance = int.Parse(maxSizeInMegabytesElement.Value, CultureInfo.InvariantCulture);
                                            entryInstance.MaxSizeInMegabytes = maxSizeInMegabytesInstance;
                                        }
                                        
                                        XElement requiresDuplicateDetectionElement = queueDescriptionElement.Element(XName.Get("RequiresDuplicateDetection", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (requiresDuplicateDetectionElement != null && requiresDuplicateDetectionElement.IsEmpty == false)
                                        {
                                            bool requiresDuplicateDetectionInstance = bool.Parse(requiresDuplicateDetectionElement.Value);
                                            entryInstance.RequiresDuplicateDetection = requiresDuplicateDetectionInstance;
                                        }
                                        
                                        XElement requiresSessionElement = queueDescriptionElement.Element(XName.Get("RequiresSession", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (requiresSessionElement != null && requiresSessionElement.IsEmpty == false)
                                        {
                                            bool requiresSessionInstance = bool.Parse(requiresSessionElement.Value);
                                            entryInstance.RequiresSession = requiresSessionInstance;
                                        }
                                        
                                        XElement defaultMessageTimeToLiveElement = queueDescriptionElement.Element(XName.Get("DefaultMessageTimeToLive", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (defaultMessageTimeToLiveElement != null && defaultMessageTimeToLiveElement.IsEmpty == false)
                                        {
                                            string defaultMessageTimeToLiveInstance = defaultMessageTimeToLiveElement.Value;
                                            entryInstance.DefaultMessageTimeToLive = defaultMessageTimeToLiveInstance;
                                        }
                                        
                                        XElement deadLetteringOnMessageExpirationElement = queueDescriptionElement.Element(XName.Get("DeadLetteringOnMessageExpiration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (deadLetteringOnMessageExpirationElement != null && deadLetteringOnMessageExpirationElement.IsEmpty == false)
                                        {
                                            bool deadLetteringOnMessageExpirationInstance = bool.Parse(deadLetteringOnMessageExpirationElement.Value);
                                            entryInstance.DeadLetteringOnMessageExpiration = deadLetteringOnMessageExpirationInstance;
                                        }
                                        
                                        XElement duplicateDetectionHistoryTimeWindowElement = queueDescriptionElement.Element(XName.Get("DuplicateDetectionHistoryTimeWindow", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (duplicateDetectionHistoryTimeWindowElement != null && duplicateDetectionHistoryTimeWindowElement.IsEmpty == false)
                                        {
                                            string duplicateDetectionHistoryTimeWindowInstance = duplicateDetectionHistoryTimeWindowElement.Value;
                                            entryInstance.DuplicateDetectionHistoryTimeWindow = duplicateDetectionHistoryTimeWindowInstance;
                                        }
                                        
                                        XElement maxDeliveryCountElement = queueDescriptionElement.Element(XName.Get("MaxDeliveryCount", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (maxDeliveryCountElement != null && maxDeliveryCountElement.IsEmpty == false)
                                        {
                                            int maxDeliveryCountInstance = int.Parse(maxDeliveryCountElement.Value, CultureInfo.InvariantCulture);
                                            entryInstance.MaxDeliveryCount = maxDeliveryCountInstance;
                                        }
                                        
                                        XElement enableBatchedOperationsElement = queueDescriptionElement.Element(XName.Get("EnableBatchedOperations", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (enableBatchedOperationsElement != null && enableBatchedOperationsElement.IsEmpty == false)
                                        {
                                            bool enableBatchedOperationsInstance = bool.Parse(enableBatchedOperationsElement.Value);
                                            entryInstance.EnableBatchedOperations = enableBatchedOperationsInstance;
                                        }
                                        
                                        XElement sizeInBytesElement = queueDescriptionElement.Element(XName.Get("SizeInBytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (sizeInBytesElement != null && sizeInBytesElement.IsEmpty == false)
                                        {
                                            int sizeInBytesInstance = int.Parse(sizeInBytesElement.Value, CultureInfo.InvariantCulture);
                                            entryInstance.SizeInBytes = sizeInBytesInstance;
                                        }
                                        
                                        XElement messageCountElement = queueDescriptionElement.Element(XName.Get("MessageCount", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (messageCountElement != null && messageCountElement.IsEmpty == false)
                                        {
                                            int messageCountInstance = int.Parse(messageCountElement.Value, CultureInfo.InvariantCulture);
                                            entryInstance.MessageCount = messageCountInstance;
                                        }
                                        
                                        XElement isAnonymousAccessibleElement = queueDescriptionElement.Element(XName.Get("IsAnonymousAccessible", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (isAnonymousAccessibleElement != null && isAnonymousAccessibleElement.IsEmpty == false)
                                        {
                                            bool isAnonymousAccessibleInstance = bool.Parse(isAnonymousAccessibleElement.Value);
                                            entryInstance.IsAnonymousAccessible = isAnonymousAccessibleInstance;
                                        }
                                        
                                        XElement authorizationRulesSequenceElement = queueDescriptionElement.Element(XName.Get("AuthorizationRules", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (authorizationRulesSequenceElement != null && authorizationRulesSequenceElement.IsEmpty == false)
                                        {
                                            foreach (XElement authorizationRulesElement in authorizationRulesSequenceElement.Elements(XName.Get("AuthorizationRule", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect")))
                                            {
                                                ServiceBusSharedAccessAuthorizationRule authorizationRuleInstance = new ServiceBusSharedAccessAuthorizationRule();
                                                entryInstance.AuthorizationRules.Add(authorizationRuleInstance);
                                                
                                                XElement claimTypeElement = authorizationRulesElement.Element(XName.Get("ClaimType", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                                if (claimTypeElement != null && claimTypeElement.IsEmpty == false)
                                                {
                                                    string claimTypeInstance = claimTypeElement.Value;
                                                    authorizationRuleInstance.ClaimType = claimTypeInstance;
                                                }
                                                
                                                XElement claimValueElement = authorizationRulesElement.Element(XName.Get("ClaimValue", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                                if (claimValueElement != null && claimValueElement.IsEmpty == false)
                                                {
                                                    string claimValueInstance = claimValueElement.Value;
                                                    authorizationRuleInstance.ClaimValue = claimValueInstance;
                                                }
                                                
                                                XElement rightsSequenceElement = authorizationRulesElement.Element(XName.Get("Rights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                                if (rightsSequenceElement != null && rightsSequenceElement.IsEmpty == false)
                                                {
                                                    foreach (XElement rightsElement in rightsSequenceElement.Elements(XName.Get("AccessRights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect")))
                                                    {
                                                        authorizationRuleInstance.Rights.Add(((AccessRight)Enum.Parse(typeof(AccessRight), rightsElement.Value, true)));
                                                    }
                                                }
                                                
                                                XElement createdTimeElement = authorizationRulesElement.Element(XName.Get("CreatedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                                if (createdTimeElement != null && createdTimeElement.IsEmpty == false)
                                                {
                                                    DateTime createdTimeInstance = DateTime.Parse(createdTimeElement.Value, CultureInfo.InvariantCulture);
                                                    authorizationRuleInstance.CreatedTime = createdTimeInstance;
                                                }
                                                
                                                XElement keyNameElement = authorizationRulesElement.Element(XName.Get("KeyName", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                                if (keyNameElement != null && keyNameElement.IsEmpty == false)
                                                {
                                                    string keyNameInstance = keyNameElement.Value;
                                                    authorizationRuleInstance.KeyName = keyNameInstance;
                                                }
                                                
                                                XElement modifiedTimeElement = authorizationRulesElement.Element(XName.Get("ModifiedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                                if (modifiedTimeElement != null && modifiedTimeElement.IsEmpty == false)
                                                {
                                                    DateTime modifiedTimeInstance = DateTime.Parse(modifiedTimeElement.Value, CultureInfo.InvariantCulture);
                                                    authorizationRuleInstance.ModifiedTime = modifiedTimeInstance;
                                                }
                                                
                                                XElement primaryKeyElement = authorizationRulesElement.Element(XName.Get("PrimaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                                if (primaryKeyElement != null && primaryKeyElement.IsEmpty == false)
                                                {
                                                    string primaryKeyInstance = primaryKeyElement.Value;
                                                    authorizationRuleInstance.PrimaryKey = primaryKeyInstance;
                                                }
                                                
                                                XElement secondaryKeyElement = authorizationRulesElement.Element(XName.Get("SecondaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                                if (secondaryKeyElement != null && secondaryKeyElement.IsEmpty == false)
                                                {
                                                    string secondaryKeyInstance = secondaryKeyElement.Value;
                                                    authorizationRuleInstance.SecondaryKey = secondaryKeyInstance;
                                                }
                                            }
                                        }
                                        
                                        XElement statusElement = queueDescriptionElement.Element(XName.Get("Status", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (statusElement != null && statusElement.IsEmpty == false)
                                        {
                                            string statusInstance = statusElement.Value;
                                            entryInstance.Status = statusInstance;
                                        }
                                        
                                        XElement createdAtElement = queueDescriptionElement.Element(XName.Get("CreatedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (createdAtElement != null && createdAtElement.IsEmpty == false)
                                        {
                                            DateTime createdAtInstance = DateTime.Parse(createdAtElement.Value, CultureInfo.InvariantCulture);
                                            entryInstance.CreatedAt = createdAtInstance;
                                        }
                                        
                                        XElement updatedAtElement = queueDescriptionElement.Element(XName.Get("UpdatedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (updatedAtElement != null && updatedAtElement.IsEmpty == false)
                                        {
                                            DateTime updatedAtInstance = DateTime.Parse(updatedAtElement.Value, CultureInfo.InvariantCulture);
                                            entryInstance.UpdatedAt = updatedAtInstance;
                                        }
                                        
                                        XElement accessedAtElement = queueDescriptionElement.Element(XName.Get("AccessedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (accessedAtElement != null && accessedAtElement.IsEmpty == false)
                                        {
                                            DateTime accessedAtInstance = DateTime.Parse(accessedAtElement.Value, CultureInfo.InvariantCulture);
                                            entryInstance.AccessedAt = accessedAtInstance;
                                        }
                                        
                                        XElement supportOrderingElement = queueDescriptionElement.Element(XName.Get("SupportOrdering", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (supportOrderingElement != null && supportOrderingElement.IsEmpty == false)
                                        {
                                            bool supportOrderingInstance = bool.Parse(supportOrderingElement.Value);
                                            entryInstance.SupportOrdering = supportOrderingInstance;
                                        }
                                        
                                        XElement countDetailsElement = queueDescriptionElement.Element(XName.Get("CountDetails", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (countDetailsElement != null && countDetailsElement.IsEmpty == false)
                                        {
                                            CountDetails countDetailsInstance = new CountDetails();
                                            entryInstance.CountDetails = countDetailsInstance;
                                            
                                            XElement activeMessageCountElement = countDetailsElement.Element(XName.Get("ActiveMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                            if (activeMessageCountElement != null && activeMessageCountElement.IsEmpty == false)
                                            {
                                                int activeMessageCountInstance = int.Parse(activeMessageCountElement.Value, CultureInfo.InvariantCulture);
                                                countDetailsInstance.ActiveMessageCount = activeMessageCountInstance;
                                            }
                                            
                                            XElement deadLetterMessageCountElement = countDetailsElement.Element(XName.Get("DeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                            if (deadLetterMessageCountElement != null && deadLetterMessageCountElement.IsEmpty == false)
                                            {
                                                int deadLetterMessageCountInstance = int.Parse(deadLetterMessageCountElement.Value, CultureInfo.InvariantCulture);
                                                countDetailsInstance.DeadLetterMessageCount = deadLetterMessageCountInstance;
                                            }
                                            
                                            XElement scheduledMessageCountElement = countDetailsElement.Element(XName.Get("ScheduledMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                            if (scheduledMessageCountElement != null && scheduledMessageCountElement.IsEmpty == false)
                                            {
                                                int scheduledMessageCountInstance = int.Parse(scheduledMessageCountElement.Value, CultureInfo.InvariantCulture);
                                                countDetailsInstance.ScheduledMessageCount = scheduledMessageCountInstance;
                                            }
                                            
                                            XElement transferDeadLetterMessageCountElement = countDetailsElement.Element(XName.Get("TransferDeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                            if (transferDeadLetterMessageCountElement != null && transferDeadLetterMessageCountElement.IsEmpty == false)
                                            {
                                                int transferDeadLetterMessageCountInstance = int.Parse(transferDeadLetterMessageCountElement.Value, CultureInfo.InvariantCulture);
                                                countDetailsInstance.TransferDeadLetterMessageCount = transferDeadLetterMessageCountInstance;
                                            }
                                            
                                            XElement transferMessageCountElement = countDetailsElement.Element(XName.Get("TransferMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                            if (transferMessageCountElement != null && transferMessageCountElement.IsEmpty == false)
                                            {
                                                int transferMessageCountInstance = int.Parse(transferMessageCountElement.Value, CultureInfo.InvariantCulture);
                                                countDetailsInstance.TransferMessageCount = transferMessageCountInstance;
                                            }
                                        }
                                        
                                        XElement autoDeleteOnIdleElement = queueDescriptionElement.Element(XName.Get("AutoDeleteOnIdle", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (autoDeleteOnIdleElement != null && autoDeleteOnIdleElement.IsEmpty == false)
                                        {
                                            string autoDeleteOnIdleInstance = autoDeleteOnIdleElement.Value;
                                            entryInstance.AutoDeleteOnIdle = autoDeleteOnIdleInstance;
                                        }
                                        
                                        XElement entityAvailabilityStatusElement = queueDescriptionElement.Element(XName.Get("EntityAvailabilityStatus", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (entityAvailabilityStatusElement != null && entityAvailabilityStatusElement.IsEmpty == false)
                                        {
                                            string entityAvailabilityStatusInstance = entityAvailabilityStatusElement.Value;
                                            entryInstance.EntityAvailabilityStatus = entityAvailabilityStatusInstance;
                                        }
                                    }
                                }
                            }
                        }
                    }
                    
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Updates the queue description and makes a call to update
        /// corresponding DB entries.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/jj856305.aspx
        /// for more information)
        /// </summary>
        /// <param name='namespaceName'>
        /// Required. The namespace name.
        /// </param>
        /// <param name='queue'>
        /// Required. The service bus queue.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A response to a request for a particular queue.
        /// </returns>
        public async System.Threading.Tasks.Task<Microsoft.WindowsAzure.Management.ServiceBus.Models.ServiceBusQueueResponse> UpdateAsync(string namespaceName, ServiceBusQueue queue, CancellationToken cancellationToken)
        {
            // Validate
            if (namespaceName == null)
            {
                throw new ArgumentNullException("namespaceName");
            }
            if (queue == null)
            {
                throw new ArgumentNullException("queue");
            }
            
            // Tracing
            bool shouldTrace = CloudContext.Configuration.Tracing.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = Tracing.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("namespaceName", namespaceName);
                tracingParameters.Add("queue", queue);
                Tracing.Enter(invocationId, this, "UpdateAsync", tracingParameters);
            }
            
            // Construct URL
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            string url = "/" + this.Client.Credentials.SubscriptionId + "/services/servicebus/namespaces/" + namespaceName + "/queues/" + queue.Name + "/";
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Put;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("if-match", "*");
                httpRequest.Headers.Add("type", "entry");
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                httpRequest.Headers.Add("x-process-at", "ServiceBus");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                XDocument requestDoc = new XDocument();
                
                XElement entryElement = new XElement(XName.Get("entry", "http://www.w3.org/2005/Atom"));
                requestDoc.Add(entryElement);
                
                XElement contentElement = new XElement(XName.Get("content", "http://www.w3.org/2005/Atom"));
                entryElement.Add(contentElement);
                
                XAttribute typeAttribute = new XAttribute(XName.Get("type", ""), "");
                typeAttribute.Value = "application/atom+xml;type=entry;charset=utf-8";
                contentElement.Add(typeAttribute);
                
                XElement queueDescriptionElement = new XElement(XName.Get("QueueDescription", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                contentElement.Add(queueDescriptionElement);
                
                if (queue.LockDuration != null)
                {
                    XElement lockDurationElement = new XElement(XName.Get("LockDuration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    lockDurationElement.Value = queue.LockDuration;
                    queueDescriptionElement.Add(lockDurationElement);
                }
                
                XElement maxSizeInMegabytesElement = new XElement(XName.Get("MaxSizeInMegabytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                maxSizeInMegabytesElement.Value = queue.MaxSizeInMegabytes.ToString();
                queueDescriptionElement.Add(maxSizeInMegabytesElement);
                
                XElement requiresDuplicateDetectionElement = new XElement(XName.Get("RequiresDuplicateDetection", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                requiresDuplicateDetectionElement.Value = queue.RequiresDuplicateDetection.ToString().ToLower();
                queueDescriptionElement.Add(requiresDuplicateDetectionElement);
                
                XElement requiresSessionElement = new XElement(XName.Get("RequiresSession", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                requiresSessionElement.Value = queue.RequiresSession.ToString().ToLower();
                queueDescriptionElement.Add(requiresSessionElement);
                
                if (queue.DefaultMessageTimeToLive != null)
                {
                    XElement defaultMessageTimeToLiveElement = new XElement(XName.Get("DefaultMessageTimeToLive", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    defaultMessageTimeToLiveElement.Value = queue.DefaultMessageTimeToLive;
                    queueDescriptionElement.Add(defaultMessageTimeToLiveElement);
                }
                
                XElement deadLetteringOnMessageExpirationElement = new XElement(XName.Get("DeadLetteringOnMessageExpiration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                deadLetteringOnMessageExpirationElement.Value = queue.DeadLetteringOnMessageExpiration.ToString().ToLower();
                queueDescriptionElement.Add(deadLetteringOnMessageExpirationElement);
                
                if (queue.DuplicateDetectionHistoryTimeWindow != null)
                {
                    XElement duplicateDetectionHistoryTimeWindowElement = new XElement(XName.Get("DuplicateDetectionHistoryTimeWindow", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    duplicateDetectionHistoryTimeWindowElement.Value = queue.DuplicateDetectionHistoryTimeWindow;
                    queueDescriptionElement.Add(duplicateDetectionHistoryTimeWindowElement);
                }
                
                XElement enableBatchedOperationsElement = new XElement(XName.Get("EnableBatchedOperations", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                enableBatchedOperationsElement.Value = queue.EnableBatchedOperations.ToString().ToLower();
                queueDescriptionElement.Add(enableBatchedOperationsElement);
                
                XElement sizeInBytesElement = new XElement(XName.Get("SizeInBytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                sizeInBytesElement.Value = queue.SizeInBytes.ToString();
                queueDescriptionElement.Add(sizeInBytesElement);
                
                XElement messageCountElement = new XElement(XName.Get("MessageCount", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                messageCountElement.Value = queue.MessageCount.ToString();
                queueDescriptionElement.Add(messageCountElement);
                
                XElement isAnonymousAccessibleElement = new XElement(XName.Get("IsAnonymousAccessible", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                isAnonymousAccessibleElement.Value = queue.IsAnonymousAccessible.ToString().ToLower();
                queueDescriptionElement.Add(isAnonymousAccessibleElement);
                
                if (queue.AuthorizationRules != null)
                {
                    XElement authorizationRulesSequenceElement = new XElement(XName.Get("AuthorizationRules", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    foreach (ServiceBusSharedAccessAuthorizationRule authorizationRulesItem in queue.AuthorizationRules)
                    {
                        XElement authorizationRuleElement = new XElement(XName.Get("AuthorizationRule", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                        authorizationRulesSequenceElement.Add(authorizationRuleElement);
                        
                        XAttribute typeAttribute2 = new XAttribute(XName.Get("type", "http://www.w3.org/2001/XMLSchema-instance"), "");
                        typeAttribute2.Value = "SharedAccessAuthorizationRule";
                        authorizationRuleElement.Add(typeAttribute2);
                        
                        if (authorizationRulesItem.ClaimType != null)
                        {
                            XElement claimTypeElement = new XElement(XName.Get("ClaimType", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            claimTypeElement.Value = authorizationRulesItem.ClaimType;
                            authorizationRuleElement.Add(claimTypeElement);
                        }
                        
                        if (authorizationRulesItem.ClaimValue != null)
                        {
                            XElement claimValueElement = new XElement(XName.Get("ClaimValue", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            claimValueElement.Value = authorizationRulesItem.ClaimValue;
                            authorizationRuleElement.Add(claimValueElement);
                        }
                        
                        if (authorizationRulesItem.Rights != null)
                        {
                            XElement rightsSequenceElement = new XElement(XName.Get("Rights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            foreach (AccessRight rightsItem in authorizationRulesItem.Rights)
                            {
                                XElement rightsItemElement = new XElement(XName.Get("AccessRights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                rightsItemElement.Value = rightsItem.ToString();
                                rightsSequenceElement.Add(rightsItemElement);
                            }
                            authorizationRuleElement.Add(rightsSequenceElement);
                        }
                        
                        XElement createdTimeElement = new XElement(XName.Get("CreatedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                        createdTimeElement.Value = authorizationRulesItem.CreatedTime.ToString();
                        authorizationRuleElement.Add(createdTimeElement);
                        
                        if (authorizationRulesItem.KeyName != null)
                        {
                            XElement keyNameElement = new XElement(XName.Get("KeyName", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            keyNameElement.Value = authorizationRulesItem.KeyName;
                            authorizationRuleElement.Add(keyNameElement);
                        }
                        
                        XElement modifiedTimeElement = new XElement(XName.Get("ModifiedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                        modifiedTimeElement.Value = authorizationRulesItem.ModifiedTime.ToString();
                        authorizationRuleElement.Add(modifiedTimeElement);
                        
                        if (authorizationRulesItem.PrimaryKey != null)
                        {
                            XElement primaryKeyElement = new XElement(XName.Get("PrimaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            primaryKeyElement.Value = authorizationRulesItem.PrimaryKey;
                            authorizationRuleElement.Add(primaryKeyElement);
                        }
                        
                        if (authorizationRulesItem.SecondaryKey != null)
                        {
                            XElement secondaryKeyElement = new XElement(XName.Get("SecondaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            secondaryKeyElement.Value = authorizationRulesItem.SecondaryKey;
                            authorizationRuleElement.Add(secondaryKeyElement);
                        }
                    }
                    queueDescriptionElement.Add(authorizationRulesSequenceElement);
                }
                
                if (queue.Status != null)
                {
                    XElement statusElement = new XElement(XName.Get("Status", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    statusElement.Value = queue.Status;
                    queueDescriptionElement.Add(statusElement);
                }
                
                XElement supportOrderingElement = new XElement(XName.Get("SupportOrdering", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                supportOrderingElement.Value = queue.SupportOrdering.ToString().ToLower();
                queueDescriptionElement.Add(supportOrderingElement);
                
                if (queue.CountDetails != null)
                {
                    XElement countDetailsElement = new XElement(XName.Get("CountDetails", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    queueDescriptionElement.Add(countDetailsElement);
                    
                    XElement activeMessageCountElement = new XElement(XName.Get("ActiveMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                    activeMessageCountElement.Value = queue.CountDetails.ActiveMessageCount.ToString();
                    countDetailsElement.Add(activeMessageCountElement);
                    
                    XElement deadLetterMessageCountElement = new XElement(XName.Get("DeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                    deadLetterMessageCountElement.Value = queue.CountDetails.DeadLetterMessageCount.ToString();
                    countDetailsElement.Add(deadLetterMessageCountElement);
                    
                    XElement scheduledMessageCountElement = new XElement(XName.Get("ScheduledMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                    scheduledMessageCountElement.Value = queue.CountDetails.ScheduledMessageCount.ToString();
                    countDetailsElement.Add(scheduledMessageCountElement);
                    
                    XElement transferDeadLetterMessageCountElement = new XElement(XName.Get("TransferDeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                    transferDeadLetterMessageCountElement.Value = queue.CountDetails.TransferDeadLetterMessageCount.ToString();
                    countDetailsElement.Add(transferDeadLetterMessageCountElement);
                    
                    XElement transferMessageCountElement = new XElement(XName.Get("TransferMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                    transferMessageCountElement.Value = queue.CountDetails.TransferMessageCount.ToString();
                    countDetailsElement.Add(transferMessageCountElement);
                }
                
                if (queue.AutoDeleteOnIdle != null)
                {
                    XElement autoDeleteOnIdleElement = new XElement(XName.Get("AutoDeleteOnIdle", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    autoDeleteOnIdleElement.Value = queue.AutoDeleteOnIdle;
                    queueDescriptionElement.Add(autoDeleteOnIdleElement);
                }
                
                if (queue.EntityAvailabilityStatus != null)
                {
                    XElement entityAvailabilityStatusElement = new XElement(XName.Get("EntityAvailabilityStatus", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    entityAvailabilityStatusElement.Value = queue.EntityAvailabilityStatus;
                    queueDescriptionElement.Add(entityAvailabilityStatusElement);
                }
                
                requestContent = requestDoc.ToString();
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = new MediaTypeHeaderValue("application/atom+xml");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        Tracing.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        Tracing.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false), CloudExceptionType.Xml);
                        if (shouldTrace)
                        {
                            Tracing.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ServiceBusQueueResponse result = null;
                    // Deserialize Response
                    cancellationToken.ThrowIfCancellationRequested();
                    string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                    result = new ServiceBusQueueResponse();
                    XDocument responseDoc = XDocument.Parse(responseContent);
                    
                    XElement entryElement2 = responseDoc.Element(XName.Get("entry", "http://www.w3.org/2005/Atom"));
                    if (entryElement2 != null && entryElement2.IsEmpty == false)
                    {
                        XElement titleElement = entryElement2.Element(XName.Get("title", "http://www.w3.org/2005/Atom"));
                        if (titleElement != null && titleElement.IsEmpty == false)
                        {
                        }
                        
                        XElement contentElement2 = entryElement2.Element(XName.Get("content", "http://www.w3.org/2005/Atom"));
                        if (contentElement2 != null && contentElement2.IsEmpty == false)
                        {
                            XElement queueDescriptionElement2 = contentElement2.Element(XName.Get("QueueDescription", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            if (queueDescriptionElement2 != null && queueDescriptionElement2.IsEmpty == false)
                            {
                                ServiceBusQueue queueDescriptionInstance = new ServiceBusQueue();
                                result.Queue = queueDescriptionInstance;
                                
                                XElement lockDurationElement2 = queueDescriptionElement2.Element(XName.Get("LockDuration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (lockDurationElement2 != null && lockDurationElement2.IsEmpty == false)
                                {
                                    string lockDurationInstance = lockDurationElement2.Value;
                                    queueDescriptionInstance.LockDuration = lockDurationInstance;
                                }
                                
                                XElement maxSizeInMegabytesElement2 = queueDescriptionElement2.Element(XName.Get("MaxSizeInMegabytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (maxSizeInMegabytesElement2 != null && maxSizeInMegabytesElement2.IsEmpty == false)
                                {
                                    int maxSizeInMegabytesInstance = int.Parse(maxSizeInMegabytesElement2.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.MaxSizeInMegabytes = maxSizeInMegabytesInstance;
                                }
                                
                                XElement requiresDuplicateDetectionElement2 = queueDescriptionElement2.Element(XName.Get("RequiresDuplicateDetection", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (requiresDuplicateDetectionElement2 != null && requiresDuplicateDetectionElement2.IsEmpty == false)
                                {
                                    bool requiresDuplicateDetectionInstance = bool.Parse(requiresDuplicateDetectionElement2.Value);
                                    queueDescriptionInstance.RequiresDuplicateDetection = requiresDuplicateDetectionInstance;
                                }
                                
                                XElement requiresSessionElement2 = queueDescriptionElement2.Element(XName.Get("RequiresSession", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (requiresSessionElement2 != null && requiresSessionElement2.IsEmpty == false)
                                {
                                    bool requiresSessionInstance = bool.Parse(requiresSessionElement2.Value);
                                    queueDescriptionInstance.RequiresSession = requiresSessionInstance;
                                }
                                
                                XElement defaultMessageTimeToLiveElement2 = queueDescriptionElement2.Element(XName.Get("DefaultMessageTimeToLive", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (defaultMessageTimeToLiveElement2 != null && defaultMessageTimeToLiveElement2.IsEmpty == false)
                                {
                                    string defaultMessageTimeToLiveInstance = defaultMessageTimeToLiveElement2.Value;
                                    queueDescriptionInstance.DefaultMessageTimeToLive = defaultMessageTimeToLiveInstance;
                                }
                                
                                XElement deadLetteringOnMessageExpirationElement2 = queueDescriptionElement2.Element(XName.Get("DeadLetteringOnMessageExpiration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (deadLetteringOnMessageExpirationElement2 != null && deadLetteringOnMessageExpirationElement2.IsEmpty == false)
                                {
                                    bool deadLetteringOnMessageExpirationInstance = bool.Parse(deadLetteringOnMessageExpirationElement2.Value);
                                    queueDescriptionInstance.DeadLetteringOnMessageExpiration = deadLetteringOnMessageExpirationInstance;
                                }
                                
                                XElement duplicateDetectionHistoryTimeWindowElement2 = queueDescriptionElement2.Element(XName.Get("DuplicateDetectionHistoryTimeWindow", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (duplicateDetectionHistoryTimeWindowElement2 != null && duplicateDetectionHistoryTimeWindowElement2.IsEmpty == false)
                                {
                                    string duplicateDetectionHistoryTimeWindowInstance = duplicateDetectionHistoryTimeWindowElement2.Value;
                                    queueDescriptionInstance.DuplicateDetectionHistoryTimeWindow = duplicateDetectionHistoryTimeWindowInstance;
                                }
                                
                                XElement maxDeliveryCountElement = queueDescriptionElement2.Element(XName.Get("MaxDeliveryCount", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (maxDeliveryCountElement != null && maxDeliveryCountElement.IsEmpty == false)
                                {
                                    int maxDeliveryCountInstance = int.Parse(maxDeliveryCountElement.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.MaxDeliveryCount = maxDeliveryCountInstance;
                                }
                                
                                XElement enableBatchedOperationsElement2 = queueDescriptionElement2.Element(XName.Get("EnableBatchedOperations", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (enableBatchedOperationsElement2 != null && enableBatchedOperationsElement2.IsEmpty == false)
                                {
                                    bool enableBatchedOperationsInstance = bool.Parse(enableBatchedOperationsElement2.Value);
                                    queueDescriptionInstance.EnableBatchedOperations = enableBatchedOperationsInstance;
                                }
                                
                                XElement sizeInBytesElement2 = queueDescriptionElement2.Element(XName.Get("SizeInBytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (sizeInBytesElement2 != null && sizeInBytesElement2.IsEmpty == false)
                                {
                                    int sizeInBytesInstance = int.Parse(sizeInBytesElement2.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.SizeInBytes = sizeInBytesInstance;
                                }
                                
                                XElement messageCountElement2 = queueDescriptionElement2.Element(XName.Get("MessageCount", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (messageCountElement2 != null && messageCountElement2.IsEmpty == false)
                                {
                                    int messageCountInstance = int.Parse(messageCountElement2.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.MessageCount = messageCountInstance;
                                }
                                
                                XElement isAnonymousAccessibleElement2 = queueDescriptionElement2.Element(XName.Get("IsAnonymousAccessible", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (isAnonymousAccessibleElement2 != null && isAnonymousAccessibleElement2.IsEmpty == false)
                                {
                                    bool isAnonymousAccessibleInstance = bool.Parse(isAnonymousAccessibleElement2.Value);
                                    queueDescriptionInstance.IsAnonymousAccessible = isAnonymousAccessibleInstance;
                                }
                                
                                XElement authorizationRulesSequenceElement2 = queueDescriptionElement2.Element(XName.Get("AuthorizationRules", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (authorizationRulesSequenceElement2 != null && authorizationRulesSequenceElement2.IsEmpty == false)
                                {
                                    foreach (XElement authorizationRulesElement in authorizationRulesSequenceElement2.Elements(XName.Get("AuthorizationRule", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect")))
                                    {
                                        ServiceBusSharedAccessAuthorizationRule authorizationRuleInstance = new ServiceBusSharedAccessAuthorizationRule();
                                        queueDescriptionInstance.AuthorizationRules.Add(authorizationRuleInstance);
                                        
                                        XElement claimTypeElement2 = authorizationRulesElement.Element(XName.Get("ClaimType", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (claimTypeElement2 != null && claimTypeElement2.IsEmpty == false)
                                        {
                                            string claimTypeInstance = claimTypeElement2.Value;
                                            authorizationRuleInstance.ClaimType = claimTypeInstance;
                                        }
                                        
                                        XElement claimValueElement2 = authorizationRulesElement.Element(XName.Get("ClaimValue", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (claimValueElement2 != null && claimValueElement2.IsEmpty == false)
                                        {
                                            string claimValueInstance = claimValueElement2.Value;
                                            authorizationRuleInstance.ClaimValue = claimValueInstance;
                                        }
                                        
                                        XElement rightsSequenceElement2 = authorizationRulesElement.Element(XName.Get("Rights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (rightsSequenceElement2 != null && rightsSequenceElement2.IsEmpty == false)
                                        {
                                            foreach (XElement rightsElement in rightsSequenceElement2.Elements(XName.Get("AccessRights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect")))
                                            {
                                                authorizationRuleInstance.Rights.Add(((AccessRight)Enum.Parse(typeof(AccessRight), rightsElement.Value, true)));
                                            }
                                        }
                                        
                                        XElement createdTimeElement2 = authorizationRulesElement.Element(XName.Get("CreatedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (createdTimeElement2 != null && createdTimeElement2.IsEmpty == false)
                                        {
                                            DateTime createdTimeInstance = DateTime.Parse(createdTimeElement2.Value, CultureInfo.InvariantCulture);
                                            authorizationRuleInstance.CreatedTime = createdTimeInstance;
                                        }
                                        
                                        XElement keyNameElement2 = authorizationRulesElement.Element(XName.Get("KeyName", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (keyNameElement2 != null && keyNameElement2.IsEmpty == false)
                                        {
                                            string keyNameInstance = keyNameElement2.Value;
                                            authorizationRuleInstance.KeyName = keyNameInstance;
                                        }
                                        
                                        XElement modifiedTimeElement2 = authorizationRulesElement.Element(XName.Get("ModifiedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (modifiedTimeElement2 != null && modifiedTimeElement2.IsEmpty == false)
                                        {
                                            DateTime modifiedTimeInstance = DateTime.Parse(modifiedTimeElement2.Value, CultureInfo.InvariantCulture);
                                            authorizationRuleInstance.ModifiedTime = modifiedTimeInstance;
                                        }
                                        
                                        XElement primaryKeyElement2 = authorizationRulesElement.Element(XName.Get("PrimaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (primaryKeyElement2 != null && primaryKeyElement2.IsEmpty == false)
                                        {
                                            string primaryKeyInstance = primaryKeyElement2.Value;
                                            authorizationRuleInstance.PrimaryKey = primaryKeyInstance;
                                        }
                                        
                                        XElement secondaryKeyElement2 = authorizationRulesElement.Element(XName.Get("SecondaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (secondaryKeyElement2 != null && secondaryKeyElement2.IsEmpty == false)
                                        {
                                            string secondaryKeyInstance = secondaryKeyElement2.Value;
                                            authorizationRuleInstance.SecondaryKey = secondaryKeyInstance;
                                        }
                                    }
                                }
                                
                                XElement statusElement2 = queueDescriptionElement2.Element(XName.Get("Status", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (statusElement2 != null && statusElement2.IsEmpty == false)
                                {
                                    string statusInstance = statusElement2.Value;
                                    queueDescriptionInstance.Status = statusInstance;
                                }
                                
                                XElement createdAtElement = queueDescriptionElement2.Element(XName.Get("CreatedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (createdAtElement != null && createdAtElement.IsEmpty == false)
                                {
                                    DateTime createdAtInstance = DateTime.Parse(createdAtElement.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.CreatedAt = createdAtInstance;
                                }
                                
                                XElement updatedAtElement = queueDescriptionElement2.Element(XName.Get("UpdatedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (updatedAtElement != null && updatedAtElement.IsEmpty == false)
                                {
                                    DateTime updatedAtInstance = DateTime.Parse(updatedAtElement.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.UpdatedAt = updatedAtInstance;
                                }
                                
                                XElement accessedAtElement = queueDescriptionElement2.Element(XName.Get("AccessedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (accessedAtElement != null && accessedAtElement.IsEmpty == false)
                                {
                                    DateTime accessedAtInstance = DateTime.Parse(accessedAtElement.Value, CultureInfo.InvariantCulture);
                                    queueDescriptionInstance.AccessedAt = accessedAtInstance;
                                }
                                
                                XElement supportOrderingElement2 = queueDescriptionElement2.Element(XName.Get("SupportOrdering", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (supportOrderingElement2 != null && supportOrderingElement2.IsEmpty == false)
                                {
                                    bool supportOrderingInstance = bool.Parse(supportOrderingElement2.Value);
                                    queueDescriptionInstance.SupportOrdering = supportOrderingInstance;
                                }
                                
                                XElement countDetailsElement2 = queueDescriptionElement2.Element(XName.Get("CountDetails", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (countDetailsElement2 != null && countDetailsElement2.IsEmpty == false)
                                {
                                    CountDetails countDetailsInstance = new CountDetails();
                                    queueDescriptionInstance.CountDetails = countDetailsInstance;
                                    
                                    XElement activeMessageCountElement2 = countDetailsElement2.Element(XName.Get("ActiveMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                    if (activeMessageCountElement2 != null && activeMessageCountElement2.IsEmpty == false)
                                    {
                                        int activeMessageCountInstance = int.Parse(activeMessageCountElement2.Value, CultureInfo.InvariantCulture);
                                        countDetailsInstance.ActiveMessageCount = activeMessageCountInstance;
                                    }
                                    
                                    XElement deadLetterMessageCountElement2 = countDetailsElement2.Element(XName.Get("DeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                    if (deadLetterMessageCountElement2 != null && deadLetterMessageCountElement2.IsEmpty == false)
                                    {
                                        int deadLetterMessageCountInstance = int.Parse(deadLetterMessageCountElement2.Value, CultureInfo.InvariantCulture);
                                        countDetailsInstance.DeadLetterMessageCount = deadLetterMessageCountInstance;
                                    }
                                    
                                    XElement scheduledMessageCountElement2 = countDetailsElement2.Element(XName.Get("ScheduledMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                    if (scheduledMessageCountElement2 != null && scheduledMessageCountElement2.IsEmpty == false)
                                    {
                                        int scheduledMessageCountInstance = int.Parse(scheduledMessageCountElement2.Value, CultureInfo.InvariantCulture);
                                        countDetailsInstance.ScheduledMessageCount = scheduledMessageCountInstance;
                                    }
                                    
                                    XElement transferDeadLetterMessageCountElement2 = countDetailsElement2.Element(XName.Get("TransferDeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                    if (transferDeadLetterMessageCountElement2 != null && transferDeadLetterMessageCountElement2.IsEmpty == false)
                                    {
                                        int transferDeadLetterMessageCountInstance = int.Parse(transferDeadLetterMessageCountElement2.Value, CultureInfo.InvariantCulture);
                                        countDetailsInstance.TransferDeadLetterMessageCount = transferDeadLetterMessageCountInstance;
                                    }
                                    
                                    XElement transferMessageCountElement2 = countDetailsElement2.Element(XName.Get("TransferMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                    if (transferMessageCountElement2 != null && transferMessageCountElement2.IsEmpty == false)
                                    {
                                        int transferMessageCountInstance = int.Parse(transferMessageCountElement2.Value, CultureInfo.InvariantCulture);
                                        countDetailsInstance.TransferMessageCount = transferMessageCountInstance;
                                    }
                                }
                                
                                XElement autoDeleteOnIdleElement2 = queueDescriptionElement2.Element(XName.Get("AutoDeleteOnIdle", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (autoDeleteOnIdleElement2 != null && autoDeleteOnIdleElement2.IsEmpty == false)
                                {
                                    string autoDeleteOnIdleInstance = autoDeleteOnIdleElement2.Value;
                                    queueDescriptionInstance.AutoDeleteOnIdle = autoDeleteOnIdleInstance;
                                }
                                
                                XElement entityAvailabilityStatusElement2 = queueDescriptionElement2.Element(XName.Get("EntityAvailabilityStatus", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (entityAvailabilityStatusElement2 != null && entityAvailabilityStatusElement2.IsEmpty == false)
                                {
                                    string entityAvailabilityStatusInstance = entityAvailabilityStatusElement2.Value;
                                    queueDescriptionInstance.EntityAvailabilityStatus = entityAvailabilityStatusInstance;
                                }
                            }
                        }
                    }
                    
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        Tracing.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
    }
}
