// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Common;
using Microsoft.WindowsAzure.Common.Internals;
using Microsoft.WindowsAzure.WebSitesExtensions;
using Microsoft.WindowsAzure.WebSitesExtensions.Models;

namespace Microsoft.WindowsAzure.WebSitesExtensions
{
    /// <summary>
    /// The websites extensions client manages the web sites deployments, web
    /// jobs and other extensions.
    /// </summary>
    public partial class WebSiteExtensionsClient : ServiceClient<WebSiteExtensionsClient>, IWebSiteExtensionsClient
    {
        private Uri _baseUri;
        
        /// <summary>
        /// The URI used as the base for all kudu requests.
        /// </summary>
        public Uri BaseUri
        {
            get { return this._baseUri; }
        }
        
        private BasicAuthenticationCloudCredentials _credentials;
        
        /// <summary>
        /// TBD.
        /// </summary>
        public BasicAuthenticationCloudCredentials Credentials
        {
            get { return this._credentials; }
        }
        
        private string _siteName;
        
        /// <summary>
        /// The site name.
        /// </summary>
        public string SiteName
        {
            get { return this._siteName; }
        }
        
        private IDeploymentOperations _deployments;
        
        /// <summary>
        /// Operations for managing the repositories.
        /// </summary>
        public virtual IDeploymentOperations Deployments
        {
            get { return this._deployments; }
        }
        
        private IDiagnosticOperations _diagnostics;
        
        /// <summary>
        /// Operations for managing the diagnostics settings.
        /// </summary>
        public virtual IDiagnosticOperations Diagnostics
        {
            get { return this._diagnostics; }
        }
        
        private IRepositoryOperations _repository;
        
        /// <summary>
        /// Operations for managing the repository.
        /// </summary>
        public virtual IRepositoryOperations Repository
        {
            get { return this._repository; }
        }
        
        private ISettingsOperations _settings;
        
        /// <summary>
        /// Operations for managing the settings.
        /// </summary>
        public virtual ISettingsOperations Settings
        {
            get { return this._settings; }
        }
        
        private IWebJobOperations _webJobs;
        
        /// <summary>
        /// Operations for managing the jobs.
        /// </summary>
        public virtual IWebJobOperations WebJobs
        {
            get { return this._webJobs; }
        }
        
        /// <summary>
        /// Initializes a new instance of the WebSiteExtensionsClient class.
        /// </summary>
        private WebSiteExtensionsClient()
            : base()
        {
            this._deployments = new DeploymentOperations(this);
            this._diagnostics = new DiagnosticOperations(this);
            this._repository = new RepositoryOperations(this);
            this._settings = new SettingsOperations(this);
            this._webJobs = new WebJobOperations(this);
            this.HttpClient.Timeout = TimeSpan.FromSeconds(300);
        }
        
        /// <summary>
        /// Initializes a new instance of the WebSiteExtensionsClient class.
        /// </summary>
        /// <param name='siteName'>
        /// Required. The site name.
        /// </param>
        /// <param name='credentials'>
        /// Required. TBD.
        /// </param>
        /// <param name='baseUri'>
        /// Required. The URI used as the base for all kudu requests.
        /// </param>
        public WebSiteExtensionsClient(string siteName, BasicAuthenticationCloudCredentials credentials, Uri baseUri)
            : this()
        {
            if (siteName == null)
            {
                throw new ArgumentNullException("siteName");
            }
            if (credentials == null)
            {
                throw new ArgumentNullException("credentials");
            }
            if (baseUri == null)
            {
                throw new ArgumentNullException("baseUri");
            }
            this._siteName = siteName;
            this._credentials = credentials;
            this._baseUri = baseUri;
            
            this.Credentials.InitializeServiceClient(this);
        }
        
        /// <summary>
        /// Initializes a new instance of the WebSiteExtensionsClient class.
        /// </summary>
        /// <param name='siteName'>
        /// Required. The site name.
        /// </param>
        /// <param name='credentials'>
        /// Required. TBD.
        /// </param>
        public WebSiteExtensionsClient(string siteName, BasicAuthenticationCloudCredentials credentials)
            : this()
        {
            if (siteName == null)
            {
                throw new ArgumentNullException("siteName");
            }
            if (credentials == null)
            {
                throw new ArgumentNullException("credentials");
            }
            this._siteName = siteName;
            this._credentials = credentials;
            this._baseUri = TypeConversion.TryParseUri("https://" + SiteName + ".scm.azurewebsites.net:443");
            
            this.Credentials.InitializeServiceClient(this);
        }
        
        /// <summary>
        /// Parse enum values for type WebJobType.
        /// </summary>
        /// <param name='value'>
        /// The value to parse.
        /// </param>
        /// <returns>
        /// The enum value.
        /// </returns>
        internal static WebJobType ParseWebJobType(string value)
        {
            if ("triggered".Equals(value, StringComparison.OrdinalIgnoreCase))
            {
                return WebJobType.Triggered;
            }
            if ("continuous".Equals(value, StringComparison.OrdinalIgnoreCase))
            {
                return WebJobType.Continuous;
            }
            throw new ArgumentOutOfRangeException("value");
        }
        
        /// <summary>
        /// Convert an enum of type WebJobType to a string.
        /// </summary>
        /// <param name='value'>
        /// The value to convert to a string.
        /// </param>
        /// <returns>
        /// The enum value as a string.
        /// </returns>
        internal static string WebJobTypeToString(WebJobType value)
        {
            if (value == WebJobType.Triggered)
            {
                return "triggered";
            }
            if (value == WebJobType.Continuous)
            {
                return "continuous";
            }
            throw new ArgumentOutOfRangeException("value");
        }
    }
}
